SystemOrganization addCategory: #'Animations-Animations'!
SystemOrganization addCategory: #'Animations-Canvas'!
SystemOrganization addCategory: #'Animations-Core'!
SystemOrganization addCategory: #'Animations-Morphic'!
SystemOrganization addCategory: #'Animations-Tests'!
SystemOrganization addCategory: #'Animations-Graphics'!
SystemOrganization addCategory: #'Animations-Examples'!

MorphicProject subclass: #AnimMorphicProject
	instanceVariableNames: 'animations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimMorphicProject methodsFor: 'accessing' stamp: 'mt 4/16/2015 14:34'!
animations

	^ animations ifNil: [animations := AnimAnimationRegistry new]! !

!AnimMorphicProject methodsFor: 'initialize' stamp: 'mt 4/16/2015 15:55'!
initMorphic

	super initMorphic.
	world initForProject: WorldState new.! !

!AnimMorphicProject methodsFor: 'initialize' stamp: 'mt 4/16/2015 16:39'!
initWorldState

	world initForProject: AnimWorldState new.
	self initializeProjectPreferences.! !

!AnimMorphicProject methodsFor: 'initialize' stamp: 'mt 4/16/2015 16:28'!
initialize

	super initialize.
	self initWorldState.! !

!AnimMorphicProject methodsFor: 'initialize' stamp: 'mt 4/16/2015 16:28'!
installPasteUpAsWorld: pasteUpMorph

	super installPasteUpAsWorld: pasteUpMorph.
	self initWorldState.! !

!AnimMorphicProject methodsFor: 'testing' stamp: 'mt 4/16/2015 14:20'!
isAnimatedMorphic

	^ true! !

!AnimMorphicProject methodsFor: 'active process' stamp: 'mt 4/16/2015 15:53'!
spawnNewProcess

	uiProcess := [
		[
			World doOneCycle.
		
			"Simulate Animation Framework"
			self animations updateAnimations.	
			
			Processor yield.
		] repeat.
	] newProcess priority: Processor userSchedulingPriority.
	uiProcess resume! !

!MorphicProject methodsFor: '*animations-morphic' stamp: 'mt 4/16/2015 14:32'!
isAnimatedMorphic

	^ false! !

WorldState subclass: #AnimWorldState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimWorldState methodsFor: 'canvas' stamp: 'mt 4/16/2015 17:07'!
canvas: aCanvas

	(aCanvas class == FormCanvas)  
		ifTrue: [AnimFormCanvas adoptInstance: aCanvas].

	super canvas: aCanvas.! !

Morph subclass: #AnimMorph
	instanceVariableNames: 'animatedStates currentFrame currentDirection currentState drawMode lastFrame'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimMorph class methodsFor: 'defaults' stamp: 'mt 12/4/2017 13:57'!
knownDirections
	^ (0 to: 360 by: 45) asArray! !

!AnimMorph class methodsFor: 'defaults' stamp: 'mt 12/4/2017 13:56'!
knownStates
	^ #(stopped walking running appearing disappearing bouncing
	launching jumping falling climbing crouchDown standUp)! !

!AnimMorph methodsFor: 'updating' stamp: 'mt 12/4/2017 15:19'!
animateFrame: aForm

	lastFrame == aForm ifTrue: [^ self].
	
	self drawMode = #direct ifTrue: [
		self currentFrame: aForm].
	
	self drawMode = #merged ifTrue: [
		aForm displayOn: self currentFrame at: 0@0 rule: Form paint].
	
	self drawMode = #mergedDiff ifTrue: [
		lastFrame ifNotNil: [:obsoleteDiff |
			obsoleteDiff displayOn: self currentFrame at: 0@0 rule: Form erase].
		aForm displayOn: self currentFrame at: 0@0 rule: Form paint]. 
	
	self redrawLater.
	lastFrame := aForm.! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:28'!
animatedStates
	^ animatedStates! !

!AnimMorph methodsFor: 'support' stamp: 'mt 12/4/2017 15:37'!
animationsDo: block
	
	self animatedStates keysAndValuesDo: [:state :directions |
		directions keysAndValuesDo: [:direction :animation |
			block cull: animation cull: direction cull: state]].! !

!AnimMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/6/2017 13:39'!
compose: block

	| composer |
	composer := AnimComposer new
		target: self;
		yourself.
		
	block value: composer.
	
	composer animation
		start;
		register.
		
	^ composer animation! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:50'!
currentAnimation

	^ self get: self currentState in: self currentDirection! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:48'!
currentDirection

	^ currentDirection! !

!AnimMorph methodsFor: 'accessing - private' stamp: 'mt 12/4/2017 13:48'!
currentDirection: anObject

	currentDirection := anObject! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:43'!
currentFrame
	^ currentFrame! !

!AnimMorph methodsFor: 'accessing - private' stamp: 'mt 12/4/2017 13:43'!
currentFrame: aForm
	currentFrame := aForm.! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:48'!
currentState

	^ currentState! !

!AnimMorph methodsFor: 'accessing - private' stamp: 'mt 12/4/2017 13:48'!
currentState: anObject

	currentState := anObject! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/9/2017 12:23'!
discardAnimation
	self currentAnimation stop; unregister.! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/4/2017 14:33'!
drawMode
	^ drawMode! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/4/2017 14:54'!
drawMode: aSymbol
	"#direct, #merged, #mergedDiff"
	drawMode := aSymbol.! !

!AnimMorph methodsFor: 'drawing' stamp: 'mt 12/4/2017 14:26'!
drawOn: aCanvas

	self currentFrame ifNotNil: [:form |
		aCanvas
			translucentImage: form
			at: self topLeft + form offset].! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/6/2017 08:18'!
ensureAfter: firstState always: secondState
	"Use this to chain animations such as 'stand up' then 'stop'. Call this AFTER you set up the animations for all directions."

	(self animatedStates at: firstState) valuesDo: [:anim |
		anim isInfinite ifTrue: [anim loopCount: 1].
		anim finishBlock: [self lookLike: secondState]].! !

!AnimMorph methodsFor: 'support' stamp: 'mt 12/5/2017 11:48'!
flipHorizontally: anim
	anim frames: (anim frames collect: [:form |
		(form flipBy: #horizontal centerAt: form center) in: [:newForm |
			newForm offset: form offset.
			newForm]]).! !

!AnimMorph methodsFor: 'support' stamp: 'mt 12/5/2017 11:46'!
flipVertically: anim

	anim frames: (anim frames collect: [:form |
		(form flipBy: #vertical centerAt: form center) in: [:newForm |
			newForm offset: form offset.
			newForm]]).! !

!AnimMorph methodsFor: 'accessing' stamp: 'mt 12/4/2017 13:50'!
get: state in: degrees
	"See #initialize."
	
	^ (self animatedStates at: state) at: degrees! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/5/2017 11:13'!
initialize
	

	super initialize.
	
	self initializeAnimations.
	drawMode := #direct.! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/5/2017 11:13'!
initializeAnimations

	animatedStates := OrderedDictionary new.

	self class knownStates do: [:state | | directions |
		directions := OrderedDictionary new.
		self class knownDirections do: [:deg | directions at: deg put: AnimFramesAnimation null].
		animatedStates at: state put: directions].
	
	currentState := self class knownStates first.
	currentDirection := self class knownDirections first.! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/4/2017 14:22'!
intoWorld: aWorld

	super intoWorld: aWorld.
	self resetAnimation.! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/5/2017 11:59'!
lookInto: direction

	direction = self currentDirection ifTrue: [^ self].
	self resetAnimationDuring: [self currentDirection: direction].! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/5/2017 11:59'!
lookLike: state

	state = self currentState ifTrue: [^ self].
	self resetAnimationDuring: [self currentState: state].! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/8/2017 13:27'!
lookLike: state into: direction

	(direction = self currentDirection and: [state = self currentState])
		ifTrue: [^ self].
	
	self resetAnimationDuring: [self currentDirection: direction].! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/9/2017 12:23'!
outOfWorld: aWorld

	self discardAnimation.
	super outOfWorld: aWorld.! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/4/2017 14:07'!
resetAnimation
	self resetAnimationDuring: [].! !

!AnimMorph methodsFor: 'animation control' stamp: 'mt 12/4/2017 14:37'!
resetAnimationDuring: block

	self currentAnimation
		stop;
		unregister.
	
	block value.
	
	self updateExtent.
	self resetDrawCache.
	
	self currentAnimation
		start;
		register.! !

!AnimMorph methodsFor: 'updating' stamp: 'mt 12/4/2017 14:37'!
resetDrawCache

	self drawMode = #direct ifTrue: [^ self].
	self currentFrame: (Form extent: self extent depth: 32).! !

!AnimMorph methodsFor: 'support' stamp: 'mt 12/4/2017 15:38'!
scaleBy: factor

	| targetSize |
	targetSize := (self extent * factor) rounded.
	self animationsDo: [:anim | anim frames: (anim frames collect: [:form | form scaledIntoFormOfSize: targetSize smoothing: 1])].
	self resetAnimation.! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/6/2017 08:10'!
set: state in: degrees fromAnimatedMorph: morph
	"See AnimatedImageMorph."
	
	| anim |
	anim := AnimFramesAnimation new
		name: ('{1}({2})' format: {state. degrees});
		target: self;
		frames: morph images copy;
		duration: (morph instVarNamed: #delays) sum;
		loopCount: -1;
		property: #animateFrame:;
		yourself.
	
	^ self
		set: state
		in: degrees
		to: anim! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/4/2017 14:19'!
set: state in: degrees fromFileNamed: fileName
	"GIF only at the moment."
	
	^ self
		set: state
		in: degrees
		fromAnimatedMorph: (AnimatedImageMorph fromGIFFileNamed: fileName)! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/5/2017 16:41'!
set: state in: degrees fromForms: someForms
	
	^ self
		set: state
		in: degrees
		to: ((AnimFramesAnimation
			on: self
			with: someForms)
				name: ('{1}({2})' format: {state. degrees}))! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/4/2017 14:53'!
set: state in: degrees to: anim
	"See #initialize."
	
	"To not loose track of an overwritten animation."
	self currentAnimation
		stop; unregister.
	
	(self animatedStates at: state)
		at: degrees
		put: anim.
	^ anim! !

!AnimMorph methodsFor: 'initialization' stamp: 'mt 12/6/2017 08:19'!
setFromSpriteMap: map toSpec: spec
	"Adds sprite from the map to the animations as specified. Appends new frames to animations."
	
	spec do: [:tuple |
		| sprite direction state anim |
		sprite := (map at: tuple first) at: tuple second.
		direction := tuple third.
		state := tuple fourth.
		anim := self get: state in: direction.
		
		anim
			name: ('{1}({2})' format: {state. direction});
			target: self;
			frames: (anim frames copyWith: sprite);
			duration: (anim frames size - 1) * 100 "milliseconds";
			loopCount: -1;
			property: #animateFrame:].! !

!AnimMorph methodsFor: 'updating' stamp: 'mt 12/5/2017 07:31'!
updateExtent

	self currentAnimation frames ifNotEmpty: [:frames |
		| newExtent |
		newExtent := 1@1.
		frames do: [:form | newExtent := newExtent max: form extent + form offset].
		self extent: newExtent].! !

Morph subclass: #AnimTilePainterTool
	instanceVariableNames: 'painter brushPreview mode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimTilePainterTool commentStamp: '<historical>' prior: 0!
This is a simple, interactive tools to use a tile painter. Drag-and-drop forms onto it to change the painter's brush. Toggle between #paint and #erase via the 'x' key on your keyboard. Flood fill the whole surface with a yellow/right click --- which predeces a confirmation dialog.

There is no undo at the moment.

If you are finished keep track of the painter's surface and discard both painter and painter tool.

Here is an example:

painter := AnimTilePainter new.
painter palette: (AnimTileParser parseFileNamed: 'terrain_atlas.png' skip: 32@32).
painter surface: Morph new.
painter surfaceExtent: 20@20.
painter tilesUnpacked inspect. "Drag-and-drop tiles from here."
AnimTilePainterTool openOn: painter.!

!AnimTilePainterTool class methodsFor: 'instance creation' stamp: 'mt 12/11/2017 10:51'!
openOn: painter

	^ self new
		painter: painter;
		openInWorld! !

!AnimTilePainterTool methodsFor: 'layout' stamp: 'mt 12/11/2017 12:48'!
acceptDroppingMorph: m event: e
	"Set brush."
	
	self painter brush: (m passenger isCollection ifTrue: [m passenger first] ifFalse: [m passenger]).
	self mode: #paint.
	self updateBrushPreview: e.! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:57'!
brush
	^ self painter brush! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:57'!
brushPreview

	^ brushPreview! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:57'!
brushPreview: anObject

	brushPreview := anObject! !

!AnimTilePainterTool methodsFor: 'events-processing' stamp: 'mt 12/11/2017 11:01'!
handleMouseMove: evt

	evt wasHandled
		ifFalse: [self mouseMove: evt. evt wasHandled: true].! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 11:55'!
handlesKeyboard: evt
	^ true! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 11:22'!
handlesMouseDown: evt
	^ true! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 11:07'!
handlesMouseOver: evt
	^ true! !

!AnimTilePainterTool methodsFor: 'initialization' stamp: 'mt 12/11/2017 10:51'!
initialize
	super initialize.
	
	self color: (Color gray: 0.85).
	self painter: AnimTilePainter new.! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 12:14'!
keyDown: evt

	evt keyCharacter asLowercase = $x
		ifTrue: [
			self mode = #paint ifTrue: [self mode: #erase] ifFalse: [self mode: #paint]].! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 11:12'!
margin
	^ 5@5! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 11:52'!
mode
	^ mode ifNil: [#paint]! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 12:24'!
mode: aSymbol
	"#paint, #erase"
	mode := aSymbol.
	self update: #brush.! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 12:40'!
mouseDown: evt

	self mode = #paint
		ifTrue: [self painter paintAt: (self tileGridIndexAt: evt position)].
	self mode = #erase
		ifTrue: [
			"Do not erase the ground/floor tile."
			(self painter tilesAt: (self tileGridIndexAt: evt position))
				in: [:tiles | tiles size > 1 ifTrue: [tiles first delete]]].! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 11:59'!
mouseEnter: evt
	self brushPreview show.
	evt hand newKeyboardFocus: self.! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 11:07'!
mouseLeave: evt
	self brushPreview hide.! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 12:19'!
mouseMove: evt
	self updateBrushPreview: evt.
	
	
	(evt redButtonPressed and: [evt hand submorphs isEmpty])
		ifTrue: [self mouseDown: evt].! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:43'!
painter

	^ painter! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:59'!
painter: anObject

	painter ifNotNil: [:p |
		p removeDependent: self.
		self removeAllMorphs].
	painter := anObject.
	painter ifNotNil: [:p | 
		p addDependent: self.
		self addMorph: p surface.
		self update: #surfaceExtent.
		self update: #brush].! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:46'!
surface
	^ self painter surface! !

!AnimTilePainterTool methodsFor: 'accessing' stamp: 'mt 12/11/2017 11:44'!
tileGridIndexAt: pixelPosition

	^ ((pixelPosition - self surface topLeft) // self painter tileExtent) + (1@1)! !

!AnimTilePainterTool methodsFor: 'updating' stamp: 'mt 12/11/2017 12:45'!
update: aspect

	super update: aspect.
	
	aspect = #surfaceExtent ifTrue: [
		self extent: self surface extent + (self margin * 2).
		self surface topLeft: self topLeft + self margin].

	aspect = #brush ifTrue: [ | pos |
		pos := self topLeft.
		self brushPreview ifNotNil: [:m | pos := m topLeft. m delete].
		
		self mode = #erase
			ifTrue: [
				self brushPreview: (Morph new extent: self painter tileExtent; color: (Color red alpha: 0.2); yourself).
				self brushPreview borderColor: Color red]
			ifFalse: [
				self brushPreview: self brush asMorph.
				self brushPreview borderColor: Color black].

		self brushPreview
			borderWidth: 2;
			topLeft: pos. 
			
		self addMorph: self brushPreview].! !

!AnimTilePainterTool methodsFor: 'updating' stamp: 'mt 12/11/2017 11:44'!
updateBrushPreview: evt

	(self surface bounds containsPoint: evt position)
		ifFalse: []
		ifTrue: [	self brushPreview position: (self tileGridIndexAt: evt position) - (1@1) * self painter tileExtent + self surface topLeft].! !

!AnimTilePainterTool methodsFor: 'dropping/grabbing' stamp: 'mt 12/11/2017 10:55'!
wantsDroppedMorph: m event: e

	^ m passenger isForm or: [(m passenger isCollection
		and: [m passenger notEmpty])
		and: [m passenger first isForm]]! !

!AnimTilePainterTool methodsFor: 'event handling' stamp: 'mt 12/11/2017 12:11'!
yellowButtonActivity: evt

	(UIManager default confirm: 'Do you really want to flood fill?')
		ifTrue: [self painter fill: self painter brush].! !

!Morph methodsFor: '*animations-core' stamp: 'mt 6/15/2012 10:27:03.804'!
animations

	^ self
		valueOfProperty: #animations
		ifAbsentPut: [OrderedCollection new]! !

!Morph methodsFor: '*animations-support' stamp: 'mt 11/17/2017 17:57'!
compositeAnimations

	self flag: #mutexNeeded.
	^ super compositeAnimations, (AnimAnimationRegistry value
		ifNil: [#()]
		ifNotNil: [:registry | registry compositeAnimations select: [:anim | anim includesGraphicsAnimationFor: self]])! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 5/17/2010 14:34:02.46'!
darken

	self darkenAnimation
		direction: #forward;
		start.! !

!Morph methodsFor: '*animations-support' stamp: 'mt 6/7/2010 15:24:50.774'!
darkenAnimation

	^ self animations
		detect: [:anim | anim isGraphicsAnimation and: [anim type = #brightness]]
		ifNone: [
			(AnimBrightnessAnimation
				darkenOn: self
				to: 0.2
				duration: 200)
				register]! !

!Morph methodsFor: '*animations-control' stamp: 'mt 11/17/2017 17:59'!
deleteAnimations

	self animations do: [:ea | ea unregister].
	self compositeAnimations do: [:ea | ea unregister].! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 5/17/2010 15:03:35.323'!
fadeIn

	self fadeInThen: [].! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 6/7/2010 15:24:37.806'!
fadeInThen: aBlock

	self show.
	(AnimAlphaBlendAnimation
		fadeInOn: self
		duration: 200)
		finishBlock: [aBlock value];
		register;
		start: #deleteWhenFinished.! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 5/17/2010 14:56:48.698'!
fadeOut

	self fadeOutThen: [].! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 6/7/2010 15:24:37.768'!
fadeOutThen: aBlock

	(AnimAlphaBlendAnimation
		fadeOutOn: self
		duration: 200)
		finishBlock: [self hide. aBlock value];
		register;
		start: #deleteWhenFinished.! !

!Morph methodsFor: '*animations-core' stamp: 'mt 12/4/2017 11:27'!
fullDrawAnimatedOn: aCanvas
	
	| transformedCanvas |
	self visible ifFalse: [^ self].
	(aCanvas isVisible: self fullBounds) ifFalse:[^ self].
	Project current isAnimatedMorphic ifFalse: [^ self fullDrawOn: aCanvas].
	
	(self hasProperty: #errorOnDraw)
		ifTrue:[^ self drawErrorOn: aCanvas].

	transformedCanvas := self transformedCanvasFrom: aCanvas.	
	self hasAnimationRunning
		ifTrue: [self invalidRect: (transformedCanvas clipRect intersect: self fullBounds)].
	
	self fullDrawOn: transformedCanvas.! !

!Morph methodsFor: '*animations-core' stamp: 'mt 12/4/2017 11:23'!
hasAnimationRunning
	"Use this query to schedule redraw operations."
	
	^ self animations anySatisfy: [:anim | anim isRunning]! !

!Morph methodsFor: '*animations-core' stamp: 'mt 5/17/2010 16:42:06.037'!
ignoresColorMappings

	^ self
		valueOfProperty: #ignoresColorMappings
		ifAbsent: false.! !

!Morph methodsFor: '*animations-core' stamp: 'mt 5/17/2010 16:34:56.54'!
ignoresColorMappings: aBoolean

	self
		setProperty: #ignoresColorMappings
		toValue: aBoolean.! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 5/3/2012 09:44:44.756'!
lighten

	self darkenAnimation
		direction: #backward;
		finishBlock: [self darkenAnimation unregister];
		start.! !

!Morph methodsFor: '*animations-control' stamp: 'mt 11/17/2017 17:58'!
pauseAnimations

	self animations do: [:ea | ea pause].
	self compositeAnimations do: [:ea | ea pause].! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 11/17/2017 17:51'!
pulse

	| fadeOut fadeIn pulse |
	self show.
	
	fadeOut := AnimAlphaBlendAnimation new
		morph: self;
		duration: 250;
		startValue: 1.0;
		endValue: 0.0;
		yourself.
	fadeIn := AnimAlphaBlendAnimation new
		morph: self;
		duration: 250;
		startValue: 0.0;
		endValue: 1.0;
		yourself.

	pulse := AnimCompositeAnimation new
		addAll: {fadeOut. fadeIn};
		loopCount: -1; "infinitely"
		start;
		register.! !

!Morph methodsFor: '*animations-core' stamp: 'mt 6/16/2010 12:00:04.032'!
redraw
	"Redraw this morph immediately. Calling this method from within #fullDraw or #draw may lead to infinite recursion."

	self redrawLater.
		
	World displayWorldSafely.! !

!Morph methodsFor: '*animations-core' stamp: 'mt 6/16/2010 11:59:55.64'!
redrawLater

	fullBounds 
		ifNil:[self invalidRect: self outerBounds]
		ifNotNil:[self invalidRect: fullBounds].! !

!Morph methodsFor: '*animations-control' stamp: 'mt 11/17/2017 17:58'!
resumeAnimations

	self animations do: [:ea | ea resume].
	self compositeAnimations do: [:ea | ea resume].! !

!Morph methodsFor: '*animations-control' stamp: 'mt 11/17/2017 17:58'!
stopAnimations

	self animations do: [:ea | ea stop].
	self compositeAnimations do: [:ea | ea stop].! !

!Morph methodsFor: '*animations-core' stamp: 'mt 12/4/2017 11:26'!
transformedCanvasFrom: aCanvas
	"Get the correct canvas for graphics animations."
	
	| transformedCanvas |
	self ignoresColorMappings ifTrue: [^ aCanvas originalCanvas].
	
	transformedCanvas := aCanvas.
	self animations do: [:anim |
		transformedCanvas := anim transformedCanvas: transformedCanvas].
	
	^ transformedCanvas! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 6/7/2010 15:25:32.226'!
walkToPosition: aPoint

	(AnimPropertyAnimation
		on: self
		property: #position
		start: self position
		end: aPoint
		duration: 200)
		register;
		start: #deleteWhenFinished.! !

!Morph methodsFor: '*animations-examples' stamp: 'mt 5/18/2010 17:52:00.59'!
warpToPosition: aPoint

	self fadeOutThen: [
		self
			position: aPoint;
			fadeIn].! !

FormCanvas subclass: #AnimFormCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimFormCanvas methodsFor: 'drawing-general' stamp: 'mt 4/16/2015 15:42'!
fullDrawMorph: aMorph
	aMorph fullDrawAnimatedOn: self.  ! !

!Canvas methodsFor: '*animations-core' stamp: 'mt 5/17/2010 16:35:57.946'!
originalCanvas

	^ self! !

ColorMappingCanvas subclass: #AnimColorMappingCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimColorMappingCanvas commentStamp: 'mt 5/17/2010 08:42:09.163' prior: 0!
This is an extended version of the ColorMappingCanvas that supports more functionality of a FormCanvas.!

AnimColorMappingCanvas subclass: #AnimAlphaBlendingCanvas
	instanceVariableNames: 'alpha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimAlphaBlendingCanvas class methodsFor: 'defaults' stamp: 'mt 5/17/2010 11:44:12.838'!
defaultRule

	^ Form blendAlpha! !

!AnimAlphaBlendingCanvas methodsFor: 'accessing' stamp: 'mt 5/17/2010 08:43:12.954'!
alpha

	^ alpha ifNil: [alpha := 1.0]! !

!AnimAlphaBlendingCanvas methodsFor: 'accessing' stamp: 'mt 5/17/2010 08:43:21.706'!
alpha: aFloat

	alpha := aFloat.! !

!AnimAlphaBlendingCanvas methodsFor: 'private' stamp: 'mt 6/22/2010 15:36:30.494'!
image: aForm at: aPoint sourceRect: sourceRect rule: rule

	| convertedForm applyStaticAlpha |
	convertedForm := self convertForm: aForm.
	applyStaticAlpha := convertedForm == aForm.

	applyStaticAlpha ifFalse: [
		^ myCanvas
			image: convertedForm
			at: aPoint
			sourceRect: sourceRect
			rule: rule
		].

	rule = Form blend ifTrue: [
		^ myCanvas
			image: convertedForm
			at: aPoint
			sourceRect: sourceRect
			rule: Form blendAlpha
			alpha: alpha
		].
	
	rule = Form paint ifTrue:[
		^myCanvas
			image: convertedForm
			at: aPoint
			sourceRect: sourceRect
			rule: Form paintAlpha
			alpha: alpha.
	].

	rule = Form over ifTrue:[
		^myCanvas
			image: convertedForm
			at: aPoint
			sourceRect: sourceRect
			rule: Form blendAlpha
			alpha: alpha.
	].! !

!AnimAlphaBlendingCanvas methodsFor: 'private' stamp: 'mt 5/17/2010 08:44:00.672'!
mapColor: aColor

	aColor isTransparent
		ifTrue:[^aColor].
		
	aColor isOpaque
		ifTrue:[^aColor alpha: alpha].
		
	^ aColor alpha: (aColor alpha * alpha)! !

!AnimAlphaBlendingCanvas methodsFor: 'private' stamp: 'mt 7/8/2013 11:26:47.511'!
mapFillStyle: aFillStyle

	self alpha = 1.0
		ifTrue: [^ aFillStyle].

	self flag: #performance. "mt: Expensive?"
	aFillStyle isGradientFill
		ifTrue: [^ aFillStyle copy colorRamp: (aFillStyle colorRamp collect: [:assoc | assoc key -> (self mapColor: assoc value)])].
	
	^ super mapFillStyle: aFillStyle! !

AnimColorMappingCanvas subclass: #AnimBrightnessMappingCanvas
	instanceVariableNames: 'brightness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimBrightnessMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/14/2010 18:31:07.653'!
brightness

	^ brightness ifNil: [brightness := 1.0]! !

!AnimBrightnessMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/14/2010 18:31:23.54'!
brightness: aFloat

	brightness := aFloat.! !

!AnimBrightnessMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/17/2010 16:20:07.651'!
mapColor: aColor

	^ Color
		h: aColor hue
		s: aColor saturation
		v: (aColor brightness * self brightness min: 1.0 max: 0.0)
		alpha: aColor alpha! !

!AnimColorMappingCanvas class methodsFor: 'defaults' stamp: 'mt 5/17/2010 11:13:08.779'!
defaultRule

	^ Form paint! !

!AnimColorMappingCanvas methodsFor: 'converting' stamp: 'mt 12/8/2017 16:09'!
convertForm: aForm
	"Pixel-wise color mapping for very small forms. Performance reasons."
	
	| depth |
	depth := aForm depth.
	aForm extent <= (512@512)
		ifTrue: [^ aForm collectPixels: [:pv | | tmp |
			"Fix Squeak's black/alpha speciality. See Color class >> #colorFromPixelValue:depth:."
			tmp := pv = 16rFF000000 ifTrue: [16rFF000001] ifFalse: [pv]. "black"
			depth := aForm depth.
			(self mapColor: (Color colorFromPixelValue: tmp depth: depth))
				pixelValueForDepth: depth]]. 

	^ aForm! !

!AnimColorMappingCanvas methodsFor: 'drawing-polygons' stamp: 'mt 5/17/2010 16:06:11.153'!
drawPolygon: vertices color: aColor borderWidth: bw borderColor: bc

	^ myCanvas
		drawPolygon: vertices
		color: (self mapColor: aColor)
		borderWidth: bw
		borderColor: (self mapColor: bc)! !

!AnimColorMappingCanvas methodsFor: 'drawing-polygons' stamp: 'mt 5/17/2010 16:04:29.867'!
drawPolygon: vertices fillStyle: aFillStyle borderWidth: bw borderColor: bc

	^ myCanvas
		drawPolygon: vertices
		fillStyle: (self mapFillStyle: aFillStyle)
		borderWidth: bw
		borderColor: (self mapColor: bc)! !

!AnimColorMappingCanvas methodsFor: 'drawing-rectangles' stamp: 'mt 4/21/2015 10:26'!
fillRectangle: aRectangle fillStyle: aFillStyle

	^ myCanvas
		fillRectangle: aRectangle
		fillStyle: (self mapFillStyle: aFillStyle)! !

!AnimColorMappingCanvas methodsFor: 'drawing-rectangles' stamp: 'mt 4/8/2011 12:30:19.27'!
fillRoundRect: aRectangle radius: radius fillStyle: aFillStyle

	^ myCanvas
		fillRoundRect: aRectangle
		radius: radius
		fillStyle: (self mapFillStyle: aFillStyle)! !

!AnimColorMappingCanvas methodsFor: 'drawing-rectangles' stamp: 'mt 5/17/2010 08:48:29.968'!
frameAndFillRoundRect: aRectangle radius: cornerRadius fillStyle: fillStyle borderWidth: bw borderColor: bc

	^ myCanvas
		frameAndFillRoundRect: aRectangle
		radius: cornerRadius
		fillStyle: (self mapFillStyle: fillStyle)
		borderWidth: bw
		borderColor: (self mapColor: bc)! !

!AnimColorMappingCanvas methodsFor: 'drawing-rectangles' stamp: 'mt 6/16/2010 18:11:22.96'!
frameRoundRect: aRectangle radius: radius width: borderWidth color: borderColor

	^ myCanvas
		frameRoundRect: aRectangle
		radius: radius
		width: borderWidth
		color: (self mapColor: borderColor)! !

!AnimColorMappingCanvas methodsFor: 'drawing-general' stamp: 'mt 4/21/2015 10:36'!
fullDrawMorph: aMorph
	aMorph fullDrawAnimatedOn: self.! !

!AnimColorMappingCanvas methodsFor: 'private' stamp: 'mt 6/22/2010 15:37:45.46'!
image: aForm at: aPoint sourceRect: sourceRect rule: rule

	^ myCanvas
		image: (self convertForm: aForm)
		at: aPoint
		sourceRect: sourceRect
		rule: rule.! !

!AnimColorMappingCanvas methodsFor: 'private' stamp: 'mt 5/17/2010 08:51:56.569'!
mapFillStyle: aFillStyle

	^ self mapColor: aFillStyle asColor! !

!AnimColorMappingCanvas methodsFor: 'accessing - private' stamp: 'mt 5/17/2010 16:27:35.517'!
myCanvas

	^ myCanvas! !

!AnimColorMappingCanvas methodsFor: 'accessing' stamp: 'mt 6/7/2010 15:26:00.446'!
originalCanvas
	"Return the original canvas without color mappings."
	
	| originalCanvas |
	originalCanvas := myCanvas.
	[originalCanvas isKindOf: AnimColorMappingCanvas]
		whileTrue: [originalCanvas := originalCanvas myCanvas].
	^ originalCanvas! !

!AnimColorMappingCanvas methodsFor: 'drawing' stamp: 'mt 11/16/2010 20:13'!
paragraph: paragraph bounds: bounds color: c

	| textColors oldColors |
	textColors := OrderedCollection new.
	paragraph text runs values do: [:array |
		array do: [:attr | (attr isKindOf: TextColor)
			ifTrue: [textColors addIfNotPresent: attr]]].

	oldColors := textColors collect: [:tc | tc color].
	textColors do: [:tc | tc color: (self mapColor: tc color)].

	super paragraph: paragraph bounds: bounds color: c.

	textColors withIndexDo: [:tc :i| tc color: (oldColors at: i)].! !

AnimColorMappingCanvas subclass: #AnimGrayscaleMappingCanvas
	instanceVariableNames: 'ratio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimGrayscaleMappingCanvas methodsFor: 'private' stamp: 'mt 5/15/2010 18:33:38.8'!
mapColor: aColor

	| avg |
	avg := { aColor red. aColor green. aColor blue } average.
	^ Color
		r: aColor red + ((avg - aColor red) * self ratio)
		g: aColor green + ((avg - aColor green) * self ratio)
		b: aColor blue + ((avg - aColor blue) * self ratio)
		alpha: aColor alpha! !

!AnimGrayscaleMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/15/2010 14:09:57.057'!
ratio

	^ ratio ifNil: [ratio := 1.0]! !

!AnimGrayscaleMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/15/2010 14:10:34.456'!
ratio: aFloat
	"Set the ratio between 0.0 for no change and 1.0 for grayscale."
	
	ratio := aFloat.! !

AnimColorMappingCanvas subclass: #AnimParagraphCanvas
	instanceVariableNames: 'stopIndex relative max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimParagraphCanvas methodsFor: 'drawing-text' stamp: 'mt 12/11/2017 09:00'!
drawString: s from: firstIndex to: lastIndex in: boundsRect font: fontOrNil color: c

	| actualStop scale |
	self max ~= -1
		ifTrue: [
			self max
				ifNil: [self max: lastIndex]
				ifNotNil: [:currentMax | self max: (currentMax max: lastIndex)].
			scale := self max / (lastIndex max: 1)]
		ifFalse: [
			scale := 1.0].
		
	actualStop := self relative
		ifFalse: [(self stopIndex * scale) rounded min: lastIndex]
		ifTrue: [(lastIndex * self stopIndex * scale) rounded].
		
	super drawString: s from: firstIndex to: (actualStop min: s size) in: boundsRect font: fontOrNil color: c.! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:44'!
max
	^ max! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:44'!
max: anInteger
	max := anInteger.! !

!AnimParagraphCanvas methodsFor: 'drawing' stamp: 'mt 12/11/2017 08:54'!
paragraph: p bounds: b color: c
	"Manipulate text line endings. DO NOT change canvas state here because the canvas might be re-used for several drawing operations in the graphical hierarchy."
	
	| oldLines found actualStop scale |
	self max ~= -1
		ifTrue: [
			self max
				ifNil: [self max: p text size]
				ifNotNil: [:currentMax | self max: (currentMax max: p text size)].
			scale := self max / (p text size max: 1)]
		ifFalse: [
			scale := 1.0].
		
	actualStop := self relative
		ifFalse: [(self stopIndex * scale) rounded min: p text size]
		ifTrue: [(p text size * self stopIndex * scale) rounded].
	
	actualStop >= p text size
		ifTrue: [^ super paragraph: p bounds: b color: c].
	
	found := false.
	oldLines := Array new: p lines size.
	
	p lines doWithIndex: [:line :index |
		oldLines at: index put: line last.
		found
			ifFalse: [
				actualStop > line last
					ifFalse: [
						line
							firstIndex: line first
							lastIndex: actualStop.
						found := true]]
			ifTrue: [
				line
					firstIndex: line first
					lastIndex: 0]].
		
	[super paragraph: p bounds: b color: c]
		ensure: [p lines doWithIndex: [:line :index |
			line
				firstIndex: line first
				lastIndex: (oldLines at: index)]].! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/11/2017 07:55'!
relative
	"Whether the #stopIndex should be treated relatively between 0.0 and 1.0."
	^ relative ifNil: [false]! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:01'!
relative: aBoolean
	relative := aBoolean.! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/10/2017 12:44'!
stopIndex

	^ stopIndex! !

!AnimParagraphCanvas methodsFor: 'accessing' stamp: 'mt 12/10/2017 12:44'!
stopIndex: anObject

	stopIndex := anObject! !

AnimColorMappingCanvas subclass: #AnimSaturationMappingCanvas
	instanceVariableNames: 'saturation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Canvas'!

!AnimSaturationMappingCanvas methodsFor: 'private' stamp: 'mt 5/15/2010 12:33:37.419'!
mapColor: aColor

	^ Color
		h: aColor hue
		s: (aColor saturation * self saturation min: 1.0 max: 0.0)
		v: aColor brightness
		alpha: aColor alpha! !

!AnimSaturationMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/14/2010 18:34:59.635'!
saturation

	^ saturation ifNil: [saturation := 1.0]! !

!AnimSaturationMappingCanvas methodsFor: 'accessing' stamp: 'mt 5/14/2010 18:35:10.374'!
saturation: aFloat

	saturation := aFloat.! !

TestCase subclass: #AnimTimedAnimationTest
	instanceVariableNames: 'referenceTime sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

AnimTimedAnimationTest subclass: #AnimAnimationRegistryTest
	instanceVariableNames: 'morph registry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimAnimationRegistryTest methodsFor: 'running' stamp: 'mt 4/16/2015 15:33'!
setUp

	super setUp.
	morph := Morph new.
	registry := AnimAnimationRegistry new.
	sut := AnimGraphicsAnimation new
		morph: morph;
		duration: 5000.! !

!AnimAnimationRegistryTest methodsFor: 'tests' stamp: 'mt 12/6/2017 16:22'!
test01MorphDeletion

	self assert: registry graphicsAnimations isEmpty.

	sut := AnimGraphicsAnimation new
		morph: Morph new;
		duration: 5000.
	registry register: sut.
	registry updateRegistry.
	self assert: 1 equals: registry graphicsAnimations size.
	
	Smalltalk garbageCollect.
	registry garbageCollect.
	registry updateRegistry.
	
	self assert: registry graphicsAnimations isEmpty.! !

!AnimAnimationRegistryTest methodsFor: 'tests' stamp: 'mt 12/8/2017 10:54'!
test02Registry

	self assert: registry pendingRequests isEmpty.

	sut := AnimAnimation new duration: 5000.
	registry register: sut.
	self deny: registry pendingRequests isEmpty.

	registry updateRegistry.
	self assert: registry pendingRequests isEmpty.
	self assert: 1 equals: registry otherAnimations size.! !

AnimTimedAnimationTest subclass: #AnimAnimationTest
	instanceVariableNames: 'spy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimAnimationTest methodsFor: 'running' stamp: 'mt 11/21/2017 12:00'!
setUp

	super setUp.
	sut := AnimAnimation new
		duration: 10;
		register.
	spy := AnimSignalSpy on: sut.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:50:16.816'!
testAnimationEndState
		
	sut
		duration: 50;
		loopCount: 2;
		start.
		
	self wait: 100.
	
	self
		should: sut isStopped;
		should: sut currentLoop = 2;
		should: sut currentLoopTime = 49;
		should: sut currentTime = 99.
	
	sut
		resetAfterFinished: true;
		start.
		
	self wait: 100.
	
	self
		should: sut isStopped;
		should: sut currentLoop = 1;
		should: sut currentLoopTime = 0;
		should: sut currentTime = 0.
! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:53:04.34'!
testAnimationStates
		
	self should: sut isStopped.
		
	sut
		duration: 50;
		start.
	
	self
		should: sut isRunning;
		wait: 25;
		should: sut isRunning.
		
	sut pause.
	
	self should: sut isPaused.
	
	sut resume.
	
	self
		should: sut isRunning;
		wait: 25;
		should: sut isStopped.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 12/1/2012 15:11:13.829'!
testFinishBlock
	
	| counter |
	counter := 0.
	
	sut
		duration: 50;
		finishBlock: [counter := counter + 1];
		start: #keepWhenFinished.
		
	self wait: 50.
	
	self	
		assert: sut finishedOnce;
		assert: 1 equals: counter.
		
	self wait: 50.
	
	self assert: 1 equals: counter.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:58:56.127'!
testInfiniteAnimation
	"This tests also possible time jumps."
	
	sut
		duration: 50;
		loopCount: -1;
		start.
		
	self wait: 500..
	
	self
		should: sut isRunning;
		should: sut currentLoop = 11;
		should: sut currentLoopTime = 0;
		should: sut currentTime = 500.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/14/2010 11:25:15.675'!
testOtherSignals

	sut
		duration: 50;
		loopCount: 2;
		direction: #backward.
	
	self
		should: spy lastSignal = #directionChanged;
		should: spy lastArguments = {#backward}.
		
	sut start.

	self
		wait: 51;
		should: spy lastSignal = #currentLoopChanged;
		should: spy lastArguments = 2;
		wait: 50;
		should: spy lastSignal = #finished.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:56:24.945'!
testPause
			
	sut
		duration: 50;
		start.
	
	self wait: 25.
	
	sut pause.
	
	self
		should: sut isPaused;
		should: sut currentTime = 25;
		wait: 25;
		should: sut isPaused;
		should: sut currentTime = 25.
		
	sut resume.
	
	self
		should: sut isRunning;
		should: sut currentTime = 25;
		wait: 25;
		should: sut isStopped;
		should: sut currentTime = 49.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/14/2010 11:19:06.573'!
testSignals

	sut
		duration: 50;
		start.
	
	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#stopped. #running}.
		
	sut pause.

	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#running. #paused}.
		
	sut stop.

	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#paused. #stopped}.
		
	sut
		start;
		stop.
	
	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#running. #stopped}.
! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:45:59.651'!
testSimpleAnimation

	self
		should: sut isStopped;
		should: sut currentLoop = 1;
		should: sut currentLoopTime = 0;
		should: sut currentTime = 0;
		should: sut loopCount = 1.
		
	sut
		duration: 50;
		start.
		
	self wait: 25.
	
	self
		should: sut isRunning;
		should: sut currentLoop = 1;
		should: sut currentLoopTime = 25;
		should: sut currentTime = 25.
		
	self wait: 25.
	
	self
		should: sut isStopped;
		should: sut currentLoop = 1;
		should: sut currentLoopTime = 49;
		should: sut currentTime = 49.
	
! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 12:00:51.331'!
testStartAnimation
	
	sut
		duration: 50;
		start.
	
	self wait: 25.
	
	sut start.
	
	self should: sut currentTime = 25.
	
	sut
		stop;
		start.
		
	self should: sut currentTime = 0.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 15:25'!
testStartBlock
	
	| counter |
	counter := 0.
	
	sut
		duration: 50;
		startBlock: [counter := counter + 1];
		start: #keepWhenFinished.
		
	self wait: 25.
	self assert: 1 equals: counter.
		
	sut pause.
	self wait: 25.
	self assert: 1 equals: counter.
	
	sut resume.
	self wait: 25.
	self assert: 1 equals: counter.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/14/2010 11:19:20.924'!
testStateSignals

	sut
		duration: 50;
		start.
	
	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#stopped. #running}.
		
	sut pause.

	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#running. #paused}.
		
	sut stop.

	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#paused. #stopped}.
		
	sut
		start;
		stop.
	
	self
		should: spy lastSignal = #stateChanged;
		should: spy lastArguments = {#running. #stopped}.
! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/14/2010 11:13:35.239'!
testStopAnimation
	
	sut
		duration: 50;
		start.
	
	self 
		wait: 25;
		should: sut currentTime = 25.
	
	sut stop.
		
	self should: sut currentTime = 0.! !

!AnimAnimationTest methodsFor: 'tests' stamp: 'mt 5/12/2010 11:51:01.127'!
testTotalDuration
	
	sut
		duration: 50;
		loopCount: 2.
		
	self should: sut totalDuration = 100.! !

AnimTimedAnimationTest subclass: #AnimCompositeAnimationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimCompositeAnimationTest methodsFor: 'running' stamp: 'mt 11/17/2017 16:26'!
setUp

	super setUp.
	
	sut := AnimCompositeAnimation new.
	! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 11/17/2017 16:27'!
test01Duration

	sut add: (AnimAnimation new duration: 10).
	sut add: (AnimAnimation new duration: 50).
	
	self assert: 60 equals: sut duration.! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 10:53'!
test02Running

	| a1 a2 |
	a1 := AnimAnimation new duration: 10.
	a2 := AnimAnimation new duration: 50.
	
	sut
		add: a1;
		add: a2.

	sut start.
	self assert: sut currentAnimation == a1.
	self assert: 2 equals: AnimAnimationRegistry value pendingRequests size.

	self wait: 0.
	self assert: sut isRegistered.
	self assert: a1 isRegistered.
	
	self wait: 10.	
	self assert: sut currentAnimation == a1.
	self assert: a1 isRegistered.
		
	self wait: 5.
	self assert: sut currentAnimation == a2.
	self deny: a1 isRegistered.
	self assert: a2 isRegistered.

	self assert: a1 finishedOnce.
	self assert: a2 isRunning.
	self assert: sut isRunning.

	self wait: 100.
	self assert: a2 finishedOnce.
	self wait: 5.
	self assert: sut finishedOnce.
! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 11:48'!
test03LoopTwice

	| a1 a2 |
	a1 := AnimAnimation new name: #a1; duration: 10.
	a2 := AnimAnimation new name: #a2; duration: 50.
	
	sut
		add: a1;
		add: a2;
		loopCount: 2.

	sut start.

	self wait: 20.
	self assert: sut currentAnimation == a2.
	self assert: 1 equals: sut currentLoop.
	
	self wait: 60. "Note that animation switching takes a cycle which is 'wait: 5' in this test. See #wait:."
	self assert: sut currentAnimation == a1.
	self assert: 2 equals: sut currentLoop.
	self deny: sut finishedOnce.
	
	self wait: 75.
	self assert: sut finishedOnce.
! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 12:00'!
test04LoopInfinite

	| a1 a2 switches testLoopCount |
	a1 := AnimAnimation new duration: 100.
	a2 := AnimAnimation new duration: 100.
	
	sut
		add: a1;
		add: a2;
		loopCount: -1.

	sut start.

	testLoopCount := 6.
	switches := (2 "two animations" * testLoopCount).
	self wait: ((a1 duration + a2 duration) * testLoopCount) + (switches * 5) + 55 "???".
	
	self assert: sut currentAnimation == a1.
	self assert: 7 equals: sut currentLoop.
	self deny: sut finishedOnce.! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 11/17/2017 20:16'!
test05SingleInfinitePart

	| a1 a2 a3 |
	a1 := AnimAnimation new duration: 10.
	a2 := AnimAnimation new duration: 10; loopCount: -1.
	a3 := AnimAnimation new duration: 10; loopCount: -1.
	
	sut add: a1.
	self shouldnt: [sut add: a2] raise: Error.
	self should: [sut add: a3] raise: Error.! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 11/17/2017 20:24'!
test06OnlyInfiniteEnd

	| a1 a2 a3 |
	a1 := AnimAnimation new duration: 10.
	a2 := AnimAnimation new duration: 10; loopCount: -1.
	a3 := AnimAnimation new duration: 10.
	
	sut add: a1.
	self shouldnt: [sut add: a2] raise: Error.
	self should: [sut add: a3] raise: Error.! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 11/18/2017 09:57'!
test07SingleRun

	| a1 |
	a1 := AnimAnimation new duration: 10.
	self assert: 1 equals: sut loopCount.
	
	sut add: a1.
	sut start.
	
	self assert: a1 isRunning.
	self assert: sut isRunning.
	self assert: 0 equals: a1 currentTime.

	self wait: 5.
	self assert: 5 equals: a1 currentTime.
	self assert: a1 isRunning.

	self wait: 10.
	self deny: a1 isRunning.
	self assert: a1 finishedOnce.
	self assert: 1 equals: sut currentLoop.
	
	self wait: 5.
	self deny: a1 isRunning.
	self assert: a1 finishedOnce.
	self assert: sut finishedOnce.
	self assert: 1 equals: sut currentLoop.! !

!AnimCompositeAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 12:00'!
test08Registration

	| a1 |
	a1 := AnimAnimation new duration: 10.
	sut add: a1.

	sut start.
	self wait: 0.
	
	self assert: a1 isRegistered.
	self assert: sut isRegistered.
	
	sut unregister.
	self wait: 0.
	self deny: a1 isRegistered.! !

AnimTimedAnimationTest subclass: #AnimGraphicsAnimationTest
	instanceVariableNames: 'canvas morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimGraphicsAnimationTest methodsFor: 'running' stamp: 'mt 5/15/2010 12:00:51.951'!
setUp

	super setUp.
	morph := Morph new.
	canvas := FormCanvas extent: morph extent.! !

!AnimGraphicsAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:19'!
testAlphaBlendFadeIn

	sut := AnimAlphaBlendAnimation fadeInOn: morph duration: 50.
	sut start; register.
	
	self
		wait: 0;
		assert: 0.0 equals: (sut transformedCanvas: canvas) alpha;
		wait: 25;
		assert: 0.5 equals: ((sut transformedCanvas: canvas) alpha roundTo: 0.1);
		wait: 25;
		assert: 1.0 equals: ((sut transformedCanvas: canvas) alpha roundTo: 0.1).! !

!AnimGraphicsAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:19'!
testAlphaBlendFadeOut

	sut := AnimAlphaBlendAnimation fadeOutOn: morph duration: 50.
	sut start; register.
	
	self
		wait: 0;
		should: (sut transformedCanvas: canvas) alpha = 1.0;
		wait: 25;
		should: ((sut transformedCanvas: canvas) alpha roundTo: 0.1) = 0.5;
		wait: 25;
		should: ((sut transformedCanvas: canvas) alpha roundTo: 0.1) = 0.0.! !

!AnimGraphicsAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:19'!
testBrightnessColorMapping

	| color |
	color := Color blue.
	sut := AnimBrightnessAnimation darkenOn: morph to: 0.5 duration: 50.
	sut start; register.
	
	self
		should: color brightness = 1.0;
		wait: 0;
		should: ((sut transformedCanvas: canvas) mapColor: color) brightness = 1.0;
		wait: 25;
		should: (((sut transformedCanvas: canvas) mapColor: color) brightness roundTo: 0.01) = 0.74;
		wait: 25;
		should: (((sut transformedCanvas: canvas) mapColor: color) brightness roundTo: 0.1) = 0.5.! !

!AnimGraphicsAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:19'!
testDarken

	sut := AnimBrightnessAnimation darkenOn: morph to: 0.5 duration: 50.
	sut start; register.
	
	self
		wait: 0;
		should: (sut transformedCanvas: canvas) brightness = 1.0;
		wait: 25;
		should: ((sut transformedCanvas: canvas) brightness roundTo: 0.01) = 0.74;
		wait: 25;
		should: ((sut transformedCanvas: canvas) brightness roundTo: 0.1) = 0.5.! !

!AnimGraphicsAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:19'!
testGrayscale

	| color mappedColor|
	color := Color r: 0.4 g: 0.6 b: 0.5 alpha: 1.0.
	sut := AnimGrayscaleAnimation grayOutOn: morph duration: 50.
	sut start; register.
	
	self wait: 50.
	
	mappedColor := (sut transformedCanvas: canvas) mapColor: color.
	
	self
		should: (mappedColor red roundTo: 0.1) = 0.5;
		should: (mappedColor green roundTo: 0.1) = 0.5;
		should: (mappedColor blue roundTo: 0.1) = 0.5.! !

AnimTimedAnimationTest subclass: #AnimPathAnimationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimPathAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 12:26'!
test01Path

	| m |
	m := Morph new.
	m topLeft: 100@100.
	
	sut := AnimPathAnimation on: m between: {0@0. 10@0. 10@10} within: 40.
	sut start; register.
	
	self assert: 100@100 equals: m topLeft.
	self assert: 2 equals: sut animations size.
	
	self wait: 10.
	self assert: 105@100 equals: m topLeft.
	self wait: 10.
	self assert: 110@100 equals: m topLeft.
	self wait: 10.
	self assert: 110@105 equals: m topLeft.
	self wait: 10.
	self assert: 110@110 equals: m topLeft.

	self assert: sut finishedOnce.! !

!AnimPathAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 14:06'!
test02ChangePath

	| m |
	m := Morph new.
	m topLeft: 100@100.
	
	sut := AnimPathAnimation on: m between: {0@0. 10@0. 10@10} within: 40.
	sut start; register.
	self wait: 20.
	
	self
		should: [sut setPath: {10@10. 100@100} duration: 1000]
		raise: Error.
		
	sut stop.
	self
		shouldnt: [sut setPath: {10@10. 100@100} duration: 1000]
		raise: Error.! !

!AnimPathAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 14:08'!
test03ReferencePoint

	sut := AnimPathAnimation
		on: Morph new
		between: {0@0. 100@100}
		within: 1000.
		
	self assert: #topLeft equals: sut prototype property.! !

!AnimPathAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 14:17'!
test04ClosePath

	| m |
	m := Morph new.
	
	sut := AnimPathAnimation
		on: m
		between: {0@0. 100@0. 100@100}
		within: 1500.
	sut closePath.
	sut start; register.
	
	self assert: 3 equals: sut animations size.
	self assert: 1500 equals: sut duration.
	
	self wait: 500.
	self assert: 100@0 equals: m topLeft.
	self wait: 500.
	self assert: 100@100 equals: m topLeft.
	self wait: 500.
	self assert: 0@0 equals: m topLeft.

	self assert: sut finishedOnce.! !

!AnimPathAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 14:59'!
test05MakeCircle

	| m |
	m := Morph new.
	m topLeft: 0@0.
	
	sut := AnimPathAnimation
		on: m
		between: {0@0. 10@10}
		within: 1500.
	sut makeCircle.
	sut start; register.

	self assert: 8 equals: sut animations size.
	self assert: 1500 equals: (sut duration roundTo: 1500).

	"We have to divide the updates so that the composite animation can give each part a chance."
	30 timesRepeat: [self wait: 75]. 
	
	self assert: 0@0 equals: m topLeft.
	self assert: sut finishedOnce.! !

AnimTimedAnimationTest subclass: #AnimPropertyAnimationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 12/6/2017 17:07'!
testFloats

	| valueHolder |
	valueHolder := ValueHolder new contents: 0.0.
	sut := (AnimPropertyAnimation
		on: valueHolder
		property: #contents
		start: 0.0
		end: 0.3234234234
		duration: 200)
		start;
		register.
	
	self wait: 300.
	
	self should: valueHolder contents = 0.3234234234.! !

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 12/8/2017 11:01'!
testFloatsReversed

	| valueHolder |
	valueHolder := ValueHolder new contents: 0.0.
	sut := (AnimPropertyAnimation
		on: valueHolder
		property: #contents
		start: 0.3234234234
		end: 0.0
		duration: 200)
		start;
		register.
	
	self wait: 300.
	
	self should: valueHolder contents = 0.0.! !

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 12:20'!
testFloatsReversed2

	| valueHolder |
	valueHolder := ValueHolder new contents: 0.0.
	sut := (AnimPropertyAnimation
		on: valueHolder
		property: #contents
		start: 0.3
		end: 0.0
		duration: 200)
		start;
		register.
	
	self wait: 500.
	
	self should: valueHolder contents = 0.0.! !

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 12:20'!
testMorphPositionChange

	| morph |
	morph := Morph new.
	sut := (AnimPropertyAnimation
		on: morph
		property: #position
		start: 0@0
		end: 6@12
		duration: 50)
		start;
		register.
		
	self
		wait: 25;
		should: morph position rounded = (3@6);
		wait: 25;
		should: morph position rounded = (6@12).! !

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 12:20'!
testMorphPositionChangeWithOffset

	| morph |
	morph := Morph new.
	sut := (AnimPropertyAnimation
		on: morph
		property: #position
		start: 0@0
		end: 6@12
		duration: 50)
		offset: 10@10;
		start;
		register.
		
	self
		wait: 25;
		should: morph position rounded = (13@16);
		wait: 25;
		should: morph position rounded = (16@22).! !

!AnimPropertyAnimationTest methodsFor: 'tests' stamp: 'mt 11/21/2017 12:20'!
testMorphPositionChangeWithOffsetBlock

	| morph |
	morph := Morph new.
	sut := (AnimPropertyAnimation
		on: morph
		property: #position
		start: 0@0
		end: 6@12
		duration: 50)
		offsetBlock: [10@10];
		start;
		register.
		
	self
		wait: 25;
		should: morph position rounded = (13@16);
		wait: 25;
		should: morph position rounded = (16@22).! !

!AnimTimedAnimationTest methodsFor: 'private' stamp: 'mt 11/21/2017 12:15'!
runCase

	AnimAnimationRegistry
		value: AnimAnimationRegistry new
		during: [super runCase].! !

!AnimTimedAnimationTest methodsFor: 'running' stamp: 'mt 12/6/2017 16:44'!
setUp

	super setUp.
	
	referenceTime := WorldState lastCycleTime.
	AnimAnimationRegistry value updateAnimations: referenceTime.! !

!AnimTimedAnimationTest methodsFor: 'support' stamp: 'mt 12/6/2017 17:06'!
wait: milliseconds
	"Wait some time in terms of the main world cycle. Split up waiting time into slices of 5 virtual milliseconds to get close to the real behavior."
	
	self waitNow: 0.
	milliseconds // 5 timesRepeat: [self waitNow: 5].! !

!AnimTimedAnimationTest methodsFor: 'support' stamp: 'mt 12/6/2017 16:45'!
waitNow: milliseconds

	referenceTime := referenceTime + milliseconds.
	AnimAnimationRegistry value updateAnimations: referenceTime.! !

AnimTimedAnimationTest subclass: #AnimVariantAnimationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimVariantAnimationTest methodsFor: 'running' stamp: 'mt 6/7/2010 15:23:51.684'!
setUp

	super setUp.
	sut := AnimVariantAnimation new.! !

!AnimVariantAnimationTest methodsFor: 'testing' stamp: 'mt 11/21/2017 12:21'!
testLinearInterpolation

	sut
		easingCurve: AnimEasingCurve linear;
		duration: 50;
		startValue: 1;
		endValue: 50;
		start;
		register.
		
	self
		should: sut currentValue rounded = 1;
		wait: 25;
		should: sut currentValue rounded = 26;
		wait: 60;
		should: sut isStopped;
		should: sut currentValue rounded = 50.! !

DynamicVariable subclass: #AnimAnimationRegistry
	instanceVariableNames: 'propertyAnimations graphicsAnimations compositeAnimations otherAnimations pendingRequests lastGCTime referenceTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

AnimAnimationRegistry class
	instanceVariableNames: 'instance'!

!AnimAnimationRegistry commentStamp: 'mt 5/17/2010 15:48:31.878' prior: 0!
Should be thread-safe.!

AnimAnimationRegistry class
	instanceVariableNames: 'instance'!

!AnimAnimationRegistry class methodsFor: 'default' stamp: 'mt 11/17/2017 16:34'!
default 

	^ self main! !

!AnimAnimationRegistry class methodsFor: 'default' stamp: 'mt 12/13/2015 14:59:04.414'!
main
	
	^ (Project current isMorphic and: [Project current isAnimatedMorphic])
		ifTrue: [Project current animations]
		ifFalse: [nil]! !

!AnimAnimationRegistry class methodsFor: 'default' stamp: 'mt 12/13/2015 14:58:51.466'!
reset
	"AnimAnimationRegistry reset."
	
	(Project current isMorphic and: [Project current isAnimatedMorphic])
		ifTrue: [Project current animations clear]! !

!AnimAnimationRegistry methodsFor: 'queries' stamp: 'mt 12/6/2017 16:11'!
animationsFor: anObject

	^ (self graphicsAnimations select: [:anim | anim morph = anObject]),
		(self propertyAnimations select: [:anim | anim target = anObject])! !

!AnimAnimationRegistry methodsFor: 'registering - private' stamp: 'mt 12/6/2017 16:09'!
clear

	"Clear local animation caches."
	self graphicsAnimations do: [:anim |
		anim morph ifNotNil: [:m | m removeProperty: #animations]].
		
	propertyAnimations := nil.
	graphicsAnimations := nil.
	compositeAnimations := nil.
	otherAnimations := nil.

	pendingAdditions := nil.
	pendingRemovals := nil.! !

!AnimAnimationRegistry methodsFor: 'accessing - private' stamp: 'mt 11/17/2017 17:48'!
compositeAnimations

	^ compositeAnimations ifNil: [compositeAnimations := OrderedCollection new]! !

!AnimAnimationRegistry methodsFor: 'queries' stamp: 'mt 5/3/2011 11:22:28.201'!
displayFPS

	(Smalltalk includesKey: #frameCountStartTime)
		ifFalse: [Smalltalk at: #frameCountStartTime put: WorldState lastCycleTime].
	(Smalltalk includesKey: #currentFrameCount)
		ifFalse: [Smalltalk at: #currentFrameCount put: 0].
	(Smalltalk includesKey: #lastFPS)
		ifFalse: [Smalltalk at: #lastFPS put: 0].

	(WorldState lastCycleTime - (Smalltalk at: #frameCountStartTime)) < 0
		ifTrue: [
			Smalltalk at: #frameCountStartTime put: WorldState lastCycleTime].

	(WorldState lastCycleTime - (Smalltalk at: #frameCountStartTime)) > 1000
		ifTrue: [
			Smalltalk
				at: #lastFPS put: (Smalltalk at: #currentFrameCount);
				at: #currentFrameCount put: 0;
				at: #frameCountStartTime put: WorldState lastCycleTime].
			
	Smalltalk at: #currentFrameCount put: (Smalltalk at: #currentFrameCount) + 1.
	(Smalltalk at: #lastFPS) asString, ' FPS' displayAt: 0@18.! !

!AnimAnimationRegistry methodsFor: 'error handling' stamp: 'mt 7/8/2013 11:18:36.105'!
emergencyClearFor: anExeption

	(Process forContext: anExeption signalerContext copyStack priority: Processor activePriority)
		debugWithTitle: anExeption description.
		
	self clear.
	self inform: 'There was an exception in the Animations framework.
All animations were cleared.

A debugger should have been appeared.'.! !

!AnimAnimationRegistry methodsFor: 'updating - private' stamp: 'mt 12/9/2017 13:07'!
garbageCollect

	self propertyAnimations
		select: [:anim | anim isValid not or: [anim finishedOnce and: [anim deletionPolicy = #deleteWhenFinished]]]
		thenDo: [:anim | self unregister: anim].
		
	self graphicsAnimations
		select: [:anim | anim isValid not or: [anim finishedOnce and: [anim deletionPolicy = #deleteWhenFinished]]]
		thenDo: [:anim | self unregister: anim].

	self compositeAnimations
		select: [:anim | 
			(anim canFinish or: [anim isValid]) not
				or: [anim finishedOnce and: [anim deletionPolicy = #deleteWhenFinished]]]
		thenDo: [:anim | self unregister: anim].

	self otherAnimations
		select: [:anim | anim finishedOnce and: [anim deletionPolicy = #deleteWhenFinished]]
		thenDo: [:anim | self unregister: anim].! !

!AnimAnimationRegistry methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 19:34:30.155'!
graphicsAnimations

	^ graphicsAnimations ifNil: [graphicsAnimations := OrderedCollection new]! !

!AnimAnimationRegistry methodsFor: 'queries' stamp: 'mt 12/6/2017 16:11'!
isRegistered: anim

	^ (((self graphicsAnimations identityIncludes: anim)
		or: [self propertyAnimations identityIncludes: anim])
		or: [self compositeAnimations identityIncludes: anim])
		or: [self otherAnimations identityIncludes: anim]! !

!AnimAnimationRegistry methodsFor: 'accessing - private' stamp: 'mt 11/12/2013 11:11:49.57'!
otherAnimations

	^ otherAnimations ifNil: [otherAnimations := OrderedCollection new]! !

!AnimAnimationRegistry methodsFor: 'accessing - private' stamp: 'mt 12/8/2017 10:47'!
pendingRequests
	^ pendingRequests ifNil: [pendingRequests := SharedQueue new]! !

!AnimAnimationRegistry methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 19:34:13.507'!
propertyAnimations

	^ propertyAnimations ifNil: [propertyAnimations := OrderedCollection new]! !

!AnimAnimationRegistry methodsFor: 'registering' stamp: 'mt 12/8/2017 10:59'!
register: anAnimation

	self pendingRequests nextPut: (MessageSend receiver: self selector: #registerNow: argument: anAnimation).! !

!AnimAnimationRegistry methodsFor: 'registering - private' stamp: 'mt 12/6/2017 16:00'!
registerNow: anAnimation

	anAnimation isPropertyAnimation
		ifTrue: [
			(self propertyAnimations includes: anAnimation)
				ifFalse: [self propertyAnimations add: anAnimation].
			^ self].
	
	anAnimation isGraphicsAnimation
		ifTrue: [
			(self graphicsAnimations includes: anAnimation)
				ifFalse: [self graphicsAnimations add: anAnimation].
			anAnimation morph ifNotNil: [:m |
				(m animations includes: anAnimation)
					ifFalse: [m animations add: anAnimation].
				m redrawLater].
			^ self].
		
	anAnimation isCompositeAnimation
		ifTrue: [
			(self compositeAnimations includes: anAnimation)
				ifFalse: [self compositeAnimations add: anAnimation].
			^ self].
	
	(self otherAnimations includes: anAnimation)
		ifFalse: [self otherAnimations add: anAnimation].! !

!AnimAnimationRegistry methodsFor: 'updating - private' stamp: 'mt 12/6/2017 16:10'!
tryGarbageCollect

	| interval |
	interval := lastGCTime
		ifNil: [5000]
		ifNotNil: [:t | (t to: DateAndTime now) duration asMilliSeconds].
	
	interval < 0
		ifTrue: [interval := 5000].
	
	interval < 5000
		ifTrue: [^ self].
		
	self garbageCollect.
	
	lastGCTime := DateAndTime now.! !

!AnimAnimationRegistry methodsFor: 'registering' stamp: 'mt 12/8/2017 11:00'!
unregister: anAnimation
	
	self pendingRequests nextPut: (MessageSend receiver: self selector: #unregisterNow: argument: anAnimation).! !

!AnimAnimationRegistry methodsFor: 'registering - private' stamp: 'mt 12/6/2017 16:01'!
unregisterNow: anAnimation

	anAnimation isPropertyAnimation
		ifTrue: [
			self propertyAnimations remove: anAnimation ifAbsent: [].
			^ self].
	
	anAnimation isGraphicsAnimation
		ifTrue: [
			self graphicsAnimations remove: anAnimation ifAbsent: [].
			anAnimation morph ifNotNil: [:m |
				m animations remove: anAnimation ifAbsent: [].
				m redrawLater].
			^ self].
	
	anAnimation isCompositeAnimation
		ifTrue: [
			self compositeAnimations remove: anAnimation ifAbsent: [].
			anAnimation currentAnimation ifNotNil: [:anim | anim unregister].
			^ self].	
	
	self otherAnimations remove: anAnimation ifAbsent: [].! !

!AnimAnimationRegistry methodsFor: 'updating' stamp: 'mt 11/21/2017 11:53'!
updateAnimations

	self updateAnimations: Time millisecondClockValue.! !

!AnimAnimationRegistry methodsFor: 'updating' stamp: 'mt 12/9/2017 10:09'!
updateAnimations: milliseconds

	self tryGarbageCollect.
	self updateRegistry.	

	[[
		referenceTime := milliseconds.
		self
			updateCurrentTime;
			updateCurrentValues.
	]
		on: Error do: [:ex | self emergencyClearFor: ex]]
		on: Halt do: [:ex | self emergencyClearFor: ex].! !

!AnimAnimationRegistry methodsFor: 'updating - private' stamp: 'mt 11/17/2017 17:48'!
updateCurrentTime
	"Updates the reference time for each registered animation."
	
	self propertyAnimations do: [:anim | anim updateCurrentTime: referenceTime].
	self graphicsAnimations do: [:anim | anim updateCurrentTime: referenceTime].
	self compositeAnimations do: [:anim | anim updateCurrentTime: referenceTime].
	self otherAnimations do: [:anim | anim updateCurrentTime: referenceTime].! !

!AnimAnimationRegistry methodsFor: 'updating - private' stamp: 'mt 4/16/2015 16:12'!
updateCurrentValues
	
	self propertyAnimations do: [:anim |
		anim updateCurrentValue.
		"GC here too. Needs mutex?"
		(anim target isNil or: [anim finishedOnce and: [anim deletionPolicy = #deleteWhenFinished]])
			ifTrue: [self unregister: anim]].! !

!AnimAnimationRegistry methodsFor: 'updating - private' stamp: 'mt 12/8/2017 10:51'!
updateRegistry

	| msg |
	[msg := self pendingRequests nextOrNil]
		whileNotNil: [msg value].! !

Object subclass: #AnimAnimation
	instanceVariableNames: 'deletionPolicy direction state currentLoop currentTime duration loopCount startTime pauseDuration resetAfterFinished finishedOnce finishBlock startBlock name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimAnimation commentStamp: 'mt 6/30/2011 09:59:07.207' prior: 0!
I am the abstract base class for all kinds of animations. If you use me with the world's #lastCycleTime all time, durations a.s.o. should be considered in milliseconds.

I emit the following changed-signals:

#finished ... The animation is done, e.g., after all loops are played.
#stateChanged ... Whenever the state of this animation changes. See #state:. The arguments containt the old and the new one.
#currentLoopChanged ... Whenever #currentLoop changes. Contains the current loop number.
#directionChanged ... Whenever #direction changes. Contains the new direction.

Animations use a custom ReferenceTime as "WorldState lastCycleTime" does not work for busy cycles.

Note: This is some kind of a Squeak port of the Nokia Qt 4.6 Animation Framework.!

!AnimAnimation class methodsFor: 'instance creation' stamp: 'mt 12/4/2017 13:39'!
null
	"Creates a new animation that does nothing. Note that you should share null animations due to the registry mechanism."

	^ self new
		name: '(null animation)';
		duration: 1;
		yourself! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 12:55'!
canFinish

	^ self isInfinite not! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:32:58.926'!
currentLoop

	^ currentLoop ifNil: [currentLoop := 1]! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 11:00:46.441'!
currentLoop: anInteger

	currentLoop ~= anInteger ifTrue: [
		currentLoop := anInteger.
		self changed: #currentLoopChanged with: currentLoop].! !

!AnimAnimation methodsFor: 'accessing - convenience' stamp: 'mt 12/8/2017 14:06'!
currentLoopProgress

	^ self currentLoopTime / self duration! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:47:03.583'!
currentLoopTime
	"Return the current time inside the current loop. Starts at 0 and ends at #duration."
	
	^ self currentTime \\ self duration! !

!AnimAnimation methodsFor: 'accessing - convenience' stamp: 'mt 12/8/2017 14:08'!
currentProgress

	self isInfinite ifTrue: [^ 0].
	^ (self currentLoop + self currentLoopProgress) / self loopCount! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:32:39.8'!
currentTime
	"The current time starts at 0 and ends at #totalDuration."

	^ currentTime ifNil: [currentTime := 0]! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 09:12:27.466'!
currentTime: anInteger

	currentTime := anInteger.! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 20:25:08.059'!
deletionPolicy

	^ deletionPolicy ifNil: [deletionPolicy := #deleteWhenFinished]! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 20:06:36.475'!
deletionPolicy: aPolicy
	"#keepWhenFinished, #deleteWhenFinished"
	
	deletionPolicy := aPolicy.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:32:27.874'!
direction

	^ direction ifNil: [direction := #forward]! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:23:30.9'!
direction: aDirection
	"#forward, #backward"
	
	aDirection ~=  direction ifTrue: [
		direction := aDirection.
		self changed: #directionChanged with: {aDirection}].! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 11:10:27.148'!
duration

	^ duration ifNil: [Error signal: 'Animation duration was not set!!']! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 11/17/2017 21:03'!
duration: anInteger

	duration := anInteger max: 1.! !

!AnimAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 10:15'!
evaluateFinishBlock
	self finishBlock value.! !

!AnimAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 10:15'!
evaluateStartBlock
	self startBlock value.! !

!AnimAnimation methodsFor: 'control - private' stamp: 'mt 12/9/2017 10:16'!
finish

	self state: #stopped.
	
	self resetAfterFinished
		ifTrue: [self toStart]
		ifFalse: [self toEnd].
		
	self
		finishedOnce: true;
		changed: #finished.
		
	self evaluateFinishBlock.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 12/8/2017 15:24'!
finishBlock
	
	^ finishBlock! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:13:38.918'!
finishBlock: aBlock
	"This block will be evaluated on finish."
	
	finishBlock := aBlock.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 20:17:01.832'!
finishedOnce
	"This flag is primarily used by the animation registry to perform garbage collection."

	^ finishedOnce ifNil: [finishedOnce := false]! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 20:16:04.971'!
finishedOnce: aBoolean

	finishedOnce := aBoolean.! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 11/17/2017 17:49'!
isCompositeAnimation

	^ false! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/14/2010 19:31:37.335'!
isGraphicsAnimation

	^ false! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/12/2010 10:10:10.919'!
isInfinite

	^ self loopCount = -1! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/12/2010 09:53:32.112'!
isPaused

	^ self state = #paused! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/14/2010 19:31:43.761'!
isPropertyAnimation

	^ false! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 11/17/2017 17:36'!
isRegistered

	^ AnimAnimationRegistry value
		ifNil: [false]
		ifNotNil: [:registry | registry isRegistered: self]! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/12/2010 09:53:29.451'!
isRunning

	^ self state = #running! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 5/12/2010 09:53:26.603'!
isStopped

	^ self state = #stopped! !

!AnimAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 12:59'!
isValid

	^ true! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 09:36:21.713'!
loopCount

	^ loopCount ifNil: [loopCount := 1]! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 09:42:34.762'!
loopCount: anInteger
	"Define how often the animation will be played.
	
	0 ... not played at all
	n ... run animation n times (default: 1)
	-1 ... run animation forever"

	loopCount := anInteger.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 11/18/2017 11:35'!
name
	^ name ifNil: ['(unnamed animation)']! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 11/18/2017 11:35'!
name: aString
	name := aString.! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 5/12/2010 10:35:33.769'!
pause
	"This will pause the animation. #currentTime remains unchanged until #start or #resume is called."
	
	self isRunning ifTrue: [
		self state: #paused].! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/12/2010 10:21:00.008'!
pauseDuration

	^ pauseDuration ifNil: [pauseDuration := 0]! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/12/2010 10:21:32.48'!
pauseDuration: anInteger
	"Define that part of the time that should be ignored."

	pauseDuration := anInteger.! !

!AnimAnimation methodsFor: 'printing' stamp: 'mt 12/9/2017 09:40'!
printOn: stream

		stream nextPut: $<.
		stream nextPutAll: self class name.
		stream nextPut: $>.
		stream space.

		name ifNotNil: [:nm |
			nm printOn: stream.
			stream space].

		stream nextPutAll: self state.
		stream space.
		stream nextPutAll: self direction.
		stream space.
		
		stream nextPut: $(.
		
		#(currentLoop loopCount currentLoopTime duration)
			do: [:prop | stream nextPutAll: (self perform: prop) asString]
			separatedBy: [stream space].
		
		stream nextPut: $).! !

!AnimAnimation methodsFor: 'registering' stamp: 'mt 11/17/2017 16:35'!
register

	AnimAnimationRegistry value
		ifNil: [self finish]
		ifNotNil: [:registry | registry register: self].! !

!AnimAnimation methodsFor: 'control - private' stamp: 'mt 5/15/2010 18:23:43.585'!
reset

	self
		toStart;
		finishedOnce: false;
		startTime: nil.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:50:47.79'!
resetAfterFinished

	^ resetAfterFinished ifNil: [resetAfterFinished := false]! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:51:47.207'!
resetAfterFinished: aBoolean
	"Set the animation to the beginning after it has ended. Is not used for infinite loops."

	resetAfterFinished := aBoolean.! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 5/12/2010 10:35:49.11'!
resume
	"Resumes a paused animation."
	
	self isPaused ifTrue: [
		self state: #running].! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 5/17/2010 07:45:52.436'!
reverse

	self direction = #forward
		ifTrue: [self direction: #backward]
		ifFalse: [self direction: #forward].! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 12/1/2012 15:12:58.091'!
start
	
	self start: #deleteWhenFinished.! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 5/15/2010 18:00:22.499'!
start: aDeletionPolicy
	"If the animation is paused or stopped the animation will be rewinded and start from the beginning. If the animation is running, nothing will happen."
	
	self isRunning ifFalse: [
		self
			deletionPolicy: aDeletionPolicy;
			reset;
			state: #running].! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 12/8/2017 15:23'!
startBlock
	
	^ startBlock! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 12/8/2017 15:25'!
startBlock: aBlock
	"This block will be evaluated at start."
	
	startBlock := aBlock.! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/12/2010 09:55:04.048'!
startTime

	^ startTime! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/12/2010 09:54:55.696'!
startTime: anInteger
	"Used to calculate everything."

	startTime := anInteger.! !

!AnimAnimation methodsFor: 'accessing' stamp: 'mt 5/12/2010 10:31:26.03'!
state

	^ state ifNil: [state := #stopped]! !

!AnimAnimation methodsFor: 'accessing - private' stamp: 'mt 5/12/2010 10:02:57.196'!
state: aState
	"#stopped, #paused, #running"
	
	aState ~= state ifTrue: [
		| oldState |
		oldState := state.
		state := aState.
		self changed: #stateChanged with: {oldState. aState}].! !

!AnimAnimation methodsFor: 'control' stamp: 'mt 5/12/2010 12:03:11.694'!
stop

	self
		state: #stopped;
		toStart.! !

!AnimAnimation methodsFor: 'control - private' stamp: 'mt 5/15/2010 18:23:31.947'!
toEnd

	self
		currentLoop: (self loopCount max: 1);
		currentTime: (self duration * self loopCount) - 1;
		pauseDuration: 0.! !

!AnimAnimation methodsFor: 'control - private' stamp: 'mt 5/15/2010 18:23:20.861'!
toStart

	self
		currentLoop: 1;
		currentTime: 0;
		pauseDuration: 0.! !

!AnimAnimation methodsFor: 'accessing - convenience' stamp: 'mt 5/12/2010 09:33:30.059'!
totalDuration

	^ self duration * self loopCount! !

!AnimAnimation methodsFor: 'registering' stamp: 'mt 11/17/2017 16:35'!
unregister

	AnimAnimationRegistry value
		ifNotNil: [:registry | registry unregister: self].! !

!AnimAnimation methodsFor: 'updating - private' stamp: 'mt 11/18/2017 09:53'!
updateCurrentLoop
	"Detect loop end and change. Account for jumpy clocks."
	
	| actualLoop |
	actualLoop := self currentTime // self duration.
	actualLoop >= self currentLoop
		ifTrue: [self currentLoop: actualLoop + 1].! !

!AnimAnimation methodsFor: 'updating' stamp: 'mt 12/9/2017 10:15'!
updateCurrentTime: referenceTime
	"This should be called frequently to keep the animation running."

	self isStopped ifTrue: [^ self].
	
	"Set an accurate start time in the first update cycle."
	self startTime ifNil: [
		self startTime: referenceTime.
		self evaluateStartBlock].

	self isRunning ifTrue: [
		self
			currentTime: (referenceTime - self startTime) - self pauseDuration;
			updateCurrentLoop;
			updateState].
		
	self isPaused ifTrue: [
		self pauseDuration: (referenceTime - self startTime) - self currentTime].! !

!AnimAnimation methodsFor: 'updating - private' stamp: 'mt 5/12/2010 10:55:07.045'!
updateState

	self isInfinite
		ifTrue: [^ self].
	
	self currentLoop > self loopCount
		ifTrue: [self finish]! !

AnimAnimation subclass: #AnimCompositeAnimation
	instanceVariableNames: 'animations currentIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimCompositeAnimation commentStamp: '<historical>' prior: 0!
Composite animations register automatically.!

!AnimCompositeAnimation methodsFor: 'composition' stamp: 'mt 12/11/2017 08:15'!
add: anAnimation

	anAnimation isInfinite
		ifTrue: [
			(self animations noneSatisfy: [:ea | ea isInfinite])
				ifTrue: []
				ifFalse: [Error signal: 'Only a single infinite animations possible in compositions.']]
		ifFalse: [
			(self animations anySatisfy: [:ea | ea isInfinite])
				ifTrue: [Error signal: 'You cannot add a regular animation after an infinite one.']].
	
	^ self animations add: anAnimation! !

!AnimCompositeAnimation methodsFor: 'composition' stamp: 'mt 12/11/2017 08:15'!
addAll: someAnimations

	someAnimations do: [:ea | self add: ea].
	^ someAnimations! !

!AnimCompositeAnimation methodsFor: 'accessing - private' stamp: 'mt 11/17/2017 16:27'!
animations

	^ animations ifNil: [animations := OrderedCollection new]! !

!AnimCompositeAnimation methodsFor: 'accessing - private' stamp: 'mt 11/17/2017 15:30'!
animations: anObject

	animations := anObject! !

!AnimCompositeAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 13:01'!
canFinish

	^ super canFinish and: [self animations last canFinish]! !

!AnimCompositeAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 11:32'!
changeCurrentIndex: newIndex

	| oldAnim newAnim |
	self currentIndex = newIndex ifTrue: [^ self].
	
	oldAnim := self currentAnimation.
	self currentIndex: newIndex.
	newAnim := self currentAnimation.

	self isRegistered ifTrue: [
		oldAnim unregister.
		newAnim register].! !

!AnimCompositeAnimation methodsFor: 'accessing' stamp: 'mt 12/8/2017 11:41'!
currentAnimation

	^ self animations at: self currentIndex! !

!AnimCompositeAnimation methodsFor: 'accessing - private' stamp: 'mt 12/8/2017 11:27'!
currentIndex

	^ currentIndex ifNil: [currentIndex := 1]! !

!AnimCompositeAnimation methodsFor: 'accessing - private' stamp: 'mt 12/8/2017 11:27'!
currentIndex: aNumber.

	currentIndex := aNumber.! !

!AnimCompositeAnimation methodsFor: 'accessing' stamp: 'mt 11/17/2017 15:42'!
duration

	^ (self animations collect: [:ea | ea duration]) sum! !

!AnimCompositeAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 11:41'!
hasNext

	self loopCount = -1 ifTrue: [^ true].
	self loopCount = 0 ifTrue: [^ false].

	self direction = #forward
		ifTrue: [^ (self currentIndex + 1) <= self animations size or: [self currentLoop < self loopCount]].
	
	self direction = #backward
		ifTrue: [^ (self currentIndex - 1) > 0 or: [self currentLoop < self loopCount]].! !

!AnimCompositeAnimation methodsFor: 'testing' stamp: 'mt 11/17/2017 17:54'!
includesGraphicsAnimationFor: aMorph

	^ self animations
		anySatisfy: [:anim | (anim isGraphicsAnimation and: [anim morph == aMorph])
			or: [anim isCompositeAnimation and: [anim includesGraphicsAnimationFor: aMorph]]]! !

!AnimCompositeAnimation methodsFor: 'testing' stamp: 'mt 11/17/2017 17:55'!
includesPropertyAnimationFor: aTarget

	^ self animations
		anySatisfy: [:anim | (anim isPropertyAnimation and: [anim target == aTarget])
			or: [anim isCompositeAnimation and: [anim includesPropertyAnimationFor: aTarget]]]! !

!AnimCompositeAnimation methodsFor: 'testing' stamp: 'mt 11/17/2017 17:49'!
isCompositeAnimation

	^ true! !

!AnimCompositeAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 13:00'!
isValid

	^ self animations allSatisfy: [:anim | anim isValid]! !

!AnimCompositeAnimation methodsFor: 'control' stamp: 'mt 11/17/2017 16:13'!
pause

	super pause.
	self currentAnimation pause.! !

!AnimCompositeAnimation methodsFor: 'control' stamp: 'mt 11/17/2017 16:13'!
resume

	super resume.
	self currentAnimation resume.! !

!AnimCompositeAnimation methodsFor: 'control' stamp: 'mt 12/8/2017 11:43'!
start: aDeletionPolicy

	self isRunning ifFalse: [
		self register.
		self currentAnimation register.
		
		super start: aDeletionPolicy.
		self currentAnimation start: aDeletionPolicy].! !

!AnimCompositeAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 11:39'!
startNext

	self direction = #forward
		ifTrue: [self changeCurrentIndex: self currentIndex \\ self animations size + 1].
	
	self direction = #backward
		ifTrue: [self changeCurrentIndex: self currentIndex + self animations size - 2 \\ self animations size + 1].

	self currentAnimation start: self deletionPolicy.! !

!AnimCompositeAnimation methodsFor: 'control' stamp: 'mt 12/8/2017 11:15'!
stop

	self currentAnimation stop.
	super stop.
	
	"We couple start-stop to registration because we register our animations automatically."
	self unregister.
	self currentAnimation unregister.! !

!AnimCompositeAnimation methodsFor: 'control - private' stamp: 'mt 12/8/2017 11:38'!
toEnd

	super toEnd.

	self direction = #forward
		ifTrue: [self changeCurrentIndex: self animations size].
	self direction = #backward
		ifTrue: [self changeCurrentIndex: 1].
		
	self currentAnimation toEnd.! !

!AnimCompositeAnimation methodsFor: 'control - private' stamp: 'mt 12/8/2017 11:37'!
toStart

	super toStart.

	self direction = #forward
		ifTrue: [self changeCurrentIndex: 1].
	self direction = #backward
		ifTrue: [self changeCurrentIndex: self animations size].
		
	self currentAnimation toStart.! !

!AnimCompositeAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 11:40'!
updateCurrentLoop
	
	self currentAnimation finishedOnce ifTrue: [
		self hasNext ifTrue: [
			
			self startNext.
			
			"After each cycle, increase the loop count."
			((self direction = #forward and: [self currentIndex = 1])
				or: [self direction = #backward and: [self currentIndex = self animations size]])
					ifTrue: [self currentLoop: self currentLoop + 1]]].! !

!AnimCompositeAnimation methodsFor: 'updating - private' stamp: 'mt 11/18/2017 10:29'!
updateState
	"We assume that #updateCurrentLoop is called before this. So, if we could not manage to get one of our animations running, we are finished."
	
	self currentAnimation finishedOnce
		ifTrue: [self finish].! !

AnimCompositeAnimation subclass: #AnimPathAnimation
	instanceVariableNames: 'points prototype'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimPathAnimation commentStamp: 'mt 11/21/2017 11:12' prior: 0!
Move a morph along points on a path.!

!AnimPathAnimation class methodsFor: 'examples' stamp: 'mt 11/21/2017 11:41'!
ex01move

	| m |
	m := Morph new.
	m topLeft: 100@100.
	m openInWorld.
	
	(AnimPathAnimation
		on: m
		between: {0@0. 50@50. 100@100. 100@0. 0@100. 0@0}
		within: 5000)
			register;
			start.! !

!AnimPathAnimation class methodsFor: 'examples' stamp: 'mt 11/21/2017 14:47'!
ex02circle

	| m |
	m :=  CircleMorph new.
	m extent: 20@20.
	m topLeft: 100@100.
	m openInWorld.
	
	(AnimPathAnimation
		on: m
		between: {0@0. 50@5}
		within: 5000)
			makeCircle;
			register;
			start.! !

!AnimPathAnimation class methodsFor: 'instance creation' stamp: 'mt 11/21/2017 14:04'!
on: morph between: points within: milliseconds

	^ self new
		setTarget: morph;
		setPath: points duration: milliseconds.
! !

!AnimPathAnimation methodsFor: 'initialization' stamp: 'mt 11/21/2017 14:17'!
closePath
	"Adds a point so that the end position is the same as the start position. The overall duration will not be changed."
	
	self points last ~= self points first ifTrue: [
		self
			setPath: (self points copyWith: self points first)
			duration: ((self points size - 1) * self prototype duration)].
		
	! !

!AnimPathAnimation methodsFor: 'initialization' stamp: 'mt 11/21/2017 15:11'!
makeCircle
	
	self points size ~= 2 ifTrue: [Error signal: 'You have to set exactly 2 points to extrapolate a circle.'].
	
	self makeCircle: (self points second - self points first) r.! !

!AnimPathAnimation methodsFor: 'initialization' stamp: 'mt 11/21/2017 15:11'!
makeCircle: rho
	"Extrapolates given points to form a circle."
	
	| degrees steps result lastPoint lastDegrees |
	self points size ~= 2 ifTrue: [Error signal: 'You have to set exactly 2 points to extrapolate a circle.'].
	
	degrees := (self points second - self points first) degrees.
	steps := (360.0 / degrees) ceiling.
	
	lastPoint := self points first.
	lastDegrees := degrees.
	result := OrderedCollection with: lastPoint.

	steps timesRepeat: [
		result add: lastPoint + (Point r: rho degrees: lastDegrees) rounded.
		lastPoint := result last.
		lastDegrees := lastDegrees + degrees.
		].
	
	"Always close path to bypass rounding issues."
	result last ~= result first
		ifTrue: [
			self
				setPath: result allButLast
				duration: self prototype duration.
			self closePath]
		ifFalse: [
			self
				setPath: result
				duration: self prototype duration].! !

!AnimPathAnimation methodsFor: 'accessing' stamp: 'mt 11/21/2017 11:12'!
points
	
	^ points
! !

!AnimPathAnimation methodsFor: 'accessing' stamp: 'mt 11/21/2017 14:05'!
points: somePoints

	self isStopped ifFalse: [Error signal: 'You can only change the path if the animation has stopped.'].
	somePoints size > 1 ifFalse: [Error signal: 'You have to set at least 2 points for a path.'].
	
	points := somePoints.
! !

!AnimPathAnimation methodsFor: 'accessing' stamp: 'mt 11/21/2017 12:35'!
prototype
	
	^ prototype ifNil: [prototype := AnimPropertyAnimation new property: #topLeft; yourself]
! !

!AnimPathAnimation methodsFor: 'accessing' stamp: 'mt 11/21/2017 11:12'!
prototype: anObject
	
	prototype := anObject.
! !

!AnimPathAnimation methodsFor: 'initialization' stamp: 'mt 11/21/2017 15:09'!
setPath: somePoints duration: milliseconds

	self points: somePoints.
	self prototype duration: (milliseconds // (points size - 1)).

	self updateAnimations.! !

!AnimPathAnimation methodsFor: 'initialization' stamp: 'mt 12/6/2017 14:57'!
setTarget: aMorph

	self prototype
		target: aMorph;
		offsetBlock: [aMorph topLeft].! !

!AnimPathAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 14:35'!
updateAnimations

	self isStopped ifFalse: [self stop].
	self animations removeAll.
	
	self points overlappingPairsDo: [:pt1 :pt2 |
		self add: (AnimPropertyAnimation new
			target: self prototype target;
			property: self prototype property;
			duration: self prototype duration;
			startValue: 0;
			endValue: pt2 - pt1;
			startOffsetBlock: self prototype startOffsetBlock;
			endOffsetBlock: self prototype endOffsetBlock;
			yourself)].! !

AnimAnimation subclass: #AnimVariantAnimation
	instanceVariableNames: 'easingCurve startValue endValue startOffset startOffsetBlock endOffset endOffsetBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimVariantAnimation commentStamp: 'mt 5/14/2010 14:43:20.077' prior: 0!
I extend the abstract animation with some value interpolation behaviour (besides the time). You can keep me running by calling #updateCurrentTime and #updateCurrentValue frequently.!

AnimVariantAnimation subclass: #AnimGraphicsAnimation
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimGraphicsAnimation commentStamp: 'mt 5/14/2010 16:44:02.071' prior: 0!
I am capable of graphic effects, e.g., fade-in or fade-out.!

AnimGraphicsAnimation subclass: #AnimAlphaBlendAnimation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Animations'!

!AnimAlphaBlendAnimation class methodsFor: 'examples' stamp: 'mt 6/7/2010 15:24:37.486'!
example1

	| morph |
	morph := Morph new
		position: 50@50;
		extent: 100@100.
	
	(AnimAlphaBlendAnimation fadeInOn: morph duration: 2000)
		register;
		start.
		
	morph openInWorld.! !

!AnimAlphaBlendAnimation class methodsFor: 'examples' stamp: 'mt 6/7/2010 15:24:37.578'!
example2

	| morph |
	morph := 'Hello, World!!' asText asMorph
		position: 50@50;
		extent: 100@100.
	
	(AnimAlphaBlendAnimation fadeOutOn: morph duration: 2000)
		register;
		start.
		
	morph openInWorld.! !

!AnimAlphaBlendAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 20:27:06.784'!
fadeInOn: aMorph duration: milliseconds

	^ self
		fadeInOn: aMorph
		to: 1.0
		duration: milliseconds! !

!AnimAlphaBlendAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 19:16:33.804'!
fadeInOn: aMorph to: aValue duration: milliseconds

	^ self new
		morph: aMorph;
		duration: milliseconds;
		startValue: 0.0;
		endValue: aValue! !

!AnimAlphaBlendAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 19:18:05.165'!
fadeOutOn: aMorph duration: milliseconds

	^ self
		fadeOutOn: aMorph
		to: 0.0
		duration: milliseconds! !

!AnimAlphaBlendAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 19:17:33.897'!
fadeOutOn: aMorph to: aValue duration: milliseconds

	^ self new
		morph: aMorph;
		duration: milliseconds;
		startValue: 1.0;
		endValue: aValue! !

!AnimAlphaBlendAnimation methodsFor: 'interpolating' stamp: 'mt 6/7/2010 15:26:10.967'!
transformedCanvas: aCanvas

	^ (AnimAlphaBlendingCanvas
		on: aCanvas)
		alpha: self currentValue! !

!AnimAlphaBlendAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:22:08.807'!
type

	^ #alpha! !

AnimGraphicsAnimation subclass: #AnimBrightnessAnimation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Animations'!

!AnimBrightnessAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 20:47:16.133'!
darkenOn: aMorph to: aValue duration: milliseconds
	"Example: If you want to darken the color by 50% aValue should be 0.5."

	^ self new
		morph: aMorph;
		duration: milliseconds;
		startValue: 1.0;
		endValue: 1.0 - aValue! !

!AnimBrightnessAnimation class methodsFor: 'examples' stamp: 'mt 6/7/2010 15:24:50.544'!
example1

	| morph |
	morph := Morph new
		position: 50@50;
		extent: 100@100.
	
	(AnimBrightnessAnimation darkenOn: morph to: 1.0 duration: 5000)
		register;
		start.
		
	morph openInWorld.! !

!AnimBrightnessAnimation class methodsFor: 'examples' stamp: 'mt 6/7/2010 15:24:50.588'!
example2

	| morph |
	morph := Morph new
		position: 50@50;
		extent: 100@100.
	morph addMorph: (Morph new
		position: 60@60;
		extent: 80@80;
		color: Color yellow).
	
	(AnimBrightnessAnimation darkenOn: morph to: 1.0 duration: 5000)
		start;
		register.
		
	morph openInWorld.! !

!AnimBrightnessAnimation methodsFor: 'interpolating' stamp: 'mt 6/7/2010 15:26:21.98'!
transformedCanvas: aCanvas

	^ (AnimBrightnessMappingCanvas
		on: aCanvas)
		brightness: self currentValue! !

!AnimBrightnessAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:22:19.717'!
type

	^ #brightness! !

!AnimGraphicsAnimation methodsFor: 'accessing' stamp: 'mt 12/9/2017 11:34'!
endOffset
	^ endOffset ifNil: [
		endOffsetBlock ifNotNil: [:b |
			endOffset := self morph
				ifNil: [b numArgs = 0 ifTrue: [b value]]
				ifNotNil: [:m | b cull: m]]]! !

!AnimGraphicsAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 11:12'!
evaluateFinishBlock

	self finishBlock ifNotNil: [:b |
		self morph
			ifNil: [b numArgs = 0 ifTrue: [b value]]
			ifNotNil: [:m | b cull: m]].! !

!AnimGraphicsAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 11:13'!
evaluateStartBlock

	self startBlock ifNotNil: [:b |
		self morph
			ifNil: [b numArgs = 0 ifTrue: [b value]]
			ifNotNil: [:m | b cull: m]].! !

!AnimGraphicsAnimation methodsFor: 'testing' stamp: 'mt 5/14/2010 19:32:07.36'!
isGraphicsAnimation

	^ true! !

!AnimGraphicsAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 13:00'!
isValid

	^ self morph notNil! !

!AnimGraphicsAnimation methodsFor: 'accessing' stamp: 'mt 6/16/2010 11:54:53.963'!
morph

	morph ifNil: [morph := WeakArray new: 1].
		
	^ morph at: 1.! !

!AnimGraphicsAnimation methodsFor: 'accessing' stamp: 'mt 6/16/2010 12:04:40.875'!
morph: aMorph

	morph ifNil: [morph := WeakArray new: 1].
	morph at: 1 put: aMorph.! !

!AnimGraphicsAnimation methodsFor: 'printing' stamp: 'mt 12/9/2017 09:45'!
printOn: stream

	super printOn: stream.
	
	stream space.
	self morph
		ifNotNil: [:obj | obj printOn: stream]
		ifNil: [stream nextPutAll: '<no morph>'].! !

!AnimGraphicsAnimation methodsFor: 'updating - private' stamp: 'mt 12/8/2017 14:14'!
repaint

	self morph ifNotNil: [:m | m redrawLater].! !

!AnimGraphicsAnimation methodsFor: 'control' stamp: 'mt 5/15/2010 18:16:21.399'!
resume

	super resume.
	self repaint.! !

!AnimGraphicsAnimation methodsFor: 'control' stamp: 'mt 11/18/2017 10:08'!
start
	"Keep finished animations to keep on drawing correct results for this graphics animation."

	self start: #keepWhenFinished.! !

!AnimGraphicsAnimation methodsFor: 'accessing' stamp: 'mt 12/9/2017 11:34'!
startOffset
	^ startOffset ifNil: [
		startOffsetBlock ifNotNil: [:b |
			startOffset := self morph
				ifNil: [b numArgs = 0 ifTrue: [b value]]
				ifNotNil: [:m | b cull: m]]]! !

!AnimGraphicsAnimation methodsFor: 'control - private' stamp: 'mt 11/18/2017 10:08'!
toEnd

	super toEnd.
	self repaint.! !

!AnimGraphicsAnimation methodsFor: 'control - private' stamp: 'mt 11/18/2017 10:08'!
toStart

	super toStart.
	self repaint.! !

!AnimGraphicsAnimation methodsFor: 'interpolating' stamp: 'mt 5/14/2010 18:51:16.154'!
transformedCanvas: aCanvas

	self subclassResponsibility.! !

!AnimGraphicsAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:21:57.388'!
type

	^ #graphics! !

!AnimGraphicsAnimation methodsFor: 'updating' stamp: 'mt 5/14/2010 16:49:13.97'!
updateCurrentValue: aValue
	"Do nothing here."
	! !

AnimGraphicsAnimation subclass: #AnimGrayscaleAnimation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Animations'!

!AnimGrayscaleAnimation class methodsFor: 'examples' stamp: 'mt 6/7/2010 15:25:12.287'!
example1

	| morph |
	morph := Morph new
		position: 50@50;
		extent: 100@100.
	morph addMorph: (Morph new
		position: 60@60;
		extent: 80@80;
		color: Color yellow).
	
	(AnimGrayscaleAnimation grayOutOn: morph duration: 5000)
		start;
		register.
		
	morph openInWorld.! !

!AnimGrayscaleAnimation class methodsFor: 'instance creation' stamp: 'mt 5/15/2010 14:12:30.361'!
grayOutOn: aMorph duration: milliseconds

	^ self new
		morph: aMorph;
		duration: milliseconds;
		startValue: 0.0;
		endValue: 1.0! !

!AnimGrayscaleAnimation methodsFor: 'interpolating' stamp: 'mt 6/7/2010 15:26:32.537'!
transformedCanvas: aCanvas

	^ (AnimGrayscaleMappingCanvas
		on: aCanvas)
		ratio: self currentValue! !

!AnimGrayscaleAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:22:32.926'!
type

	^ #grayscale! !

AnimGraphicsAnimation subclass: #AnimParagraphAnimation
	instanceVariableNames: 'lastCanvas scaled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimParagraphAnimation class methodsFor: 'instance creation' stamp: 'mt 12/11/2017 08:13'!
on: morph duration: milliseconds
	
	^ AnimParagraphAnimation new
		morph: morph;
		duration: milliseconds;
		startValue: 0.0; "no text"
		endValue: 1.0; "entire text"
		yourself! !

!AnimParagraphAnimation class methodsFor: 'instance creation' stamp: 'mt 12/11/2017 08:12'!
on: morph text: text duration: milliseconds
	"Animate a specific amount of characters."
	
	^ AnimParagraphAnimation new
		morph: morph;
		duration: milliseconds;
		startValue: 1;
		endValue: text size;
		yourself! !

!AnimParagraphAnimation methodsFor: 'as yet unclassified' stamp: 'mt 12/11/2017 08:56'!
scaled
	"If you apply this animation to multiple paragraphs, compute a scale so that the speed of appearance of characters is similar for texts with different lengths."
	^ scaled ifNil: [true]! !

!AnimParagraphAnimation methodsFor: 'as yet unclassified' stamp: 'mt 12/11/2017 08:56'!
scaled: aBoolean
	scaled := aBoolean.! !

!AnimParagraphAnimation methodsFor: 'control' stamp: 'mt 12/11/2017 07:53'!
start
	"Text rendering is expensive. Since the purpose of this animation is to show the entire paragraph after it has finished, we can safely delete it."
	
	self start: #deleteWhenFinished.! !

!AnimParagraphAnimation methodsFor: 'interpolating' stamp: 'mt 12/11/2017 08:58'!
transformedCanvas: aCanvas

	| lastMax |
	lastMax := self scaled ifFalse: [-1] ifTrue: [
		lastCanvas
			ifNil: [0]
			ifNotNil: [:c | c max]].
		
	lastCanvas := (AnimParagraphCanvas
		on: aCanvas)
			stopIndex: self currentValue;
			relative: (self currentStartValue + self currentEndValue) < 2;
			max: lastMax;
			yourself.
			
	^ lastCanvas! !

AnimGraphicsAnimation subclass: #AnimSaturationAnimation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Animations'!

!AnimSaturationAnimation methodsFor: 'interpolating' stamp: 'mt 6/7/2010 15:26:44.824'!
transformedCanvas: aCanvas

	^ (AnimSaturationMappingCanvas
		on: aCanvas)
		saturation: self currentValue! !

!AnimSaturationAnimation methodsFor: 'accessing' stamp: 'mt 5/17/2010 14:22:48.178'!
type

	^ #saturation! !

AnimVariantAnimation subclass: #AnimPropertyAnimation
	instanceVariableNames: 'target property'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

AnimPropertyAnimation subclass: #AnimFramesAnimation
	instanceVariableNames: 'frames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimFramesAnimation class methodsFor: 'instance creation' stamp: 'mt 12/4/2017 14:11'!
on: morph with: frames

	^ self new
		target: morph;
		frames: frames copy;
		duration: (frames size - 1) * 100 "milliseconds";
		loopCount: -1;
		property: #animateFrame:;
		yourself! !

!AnimFramesAnimation methodsFor: 'accessing' stamp: 'mt 12/4/2017 11:47'!
endValue
	^ self frames size! !

!AnimFramesAnimation methodsFor: 'accessing' stamp: 'mt 12/4/2017 14:02'!
frames
	^ frames ifNil: [#()]! !

!AnimFramesAnimation methodsFor: 'accessing' stamp: 'mt 12/4/2017 11:30'!
frames: someForms
	frames := someForms.! !

!AnimFramesAnimation methodsFor: 'accessing' stamp: 'mt 12/4/2017 11:47'!
startValue
	^ 1! !

!AnimFramesAnimation methodsFor: 'updating' stamp: 'mt 12/4/2017 11:45'!
updateCurrentValue: aValue
	"Find out about the current frame and set it. The given value is between start and end value, which corresponds to the number of frames available."
	
	self target ifNotNil: [:obj |
		obj
			perform: self property asMutator
			with: (self frames at: aValue rounded)].! !

!AnimPropertyAnimation class methodsFor: 'examples' stamp: 'mt 6/16/2010 10:39:35.692'!
example1

	| morph |
	morph := Morph new
		position: 50@50;
		extent: 100@100.
	morph addMorph: (Morph new
		position: 60@60;
		extent: 80@80;
		color: Color yellow).
	
	(AnimPropertyAnimation on: morph property: #position start: (50@50) end: (400@400) duration: 500)
		register;
		start: #deleteWhenFinished.
		
	morph openInWorld.! !

!AnimPropertyAnimation class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 16:34:42.775'!
on: anObject property: aProperty start: aStartValue end: anEndValue duration: milliseconds

	^ self new
		target: anObject;
		property: aProperty;
		startValue: aStartValue;
		endValue: anEndValue;
		duration: milliseconds! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 12/9/2017 11:34'!
endOffset
	^ endOffset ifNil: [
		endOffsetBlock ifNotNil: [:b |
			endOffset := self target
				ifNil: [b numArgs = 0 ifTrue: [b value]]
				ifNotNil: [:obj | b cull: obj]]]! !

!AnimPropertyAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 11:12'!
evaluateFinishBlock
	self finishBlock ifNotNil: [:b |
		self target
			ifNil: [b numArgs = 0 ifTrue: [b value]]
			ifNotNil: [:obj | b cull: obj]].! !

!AnimPropertyAnimation methodsFor: 'updating - private' stamp: 'mt 12/9/2017 11:13'!
evaluateStartBlock

	self startBlock ifNotNil: [:b |
		self target
			ifNil: [b numArgs = 0 ifTrue: [b value]]
			ifNotNil: [:obj | b cull: obj]].! !

!AnimPropertyAnimation methodsFor: 'testing' stamp: 'mt 5/14/2010 19:31:56.258'!
isPropertyAnimation

	^ true! !

!AnimPropertyAnimation methodsFor: 'testing' stamp: 'mt 12/9/2017 12:59'!
isValid

	^ self target notNil! !

!AnimPropertyAnimation methodsFor: 'printing' stamp: 'mt 12/9/2017 09:45'!
printOn: stream

	super printOn: stream.
	
	stream space.
	self target
		ifNotNil: [:obj | obj printOn: stream]
		ifNil: [stream nextPutAll: '<no target>'].! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 14:48:24.768'!
property

	^ property! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 16:34:33.236'!
property: aProperty

	property := aProperty.! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 12/9/2017 11:33'!
startOffset
	^ startOffset ifNil: [
		startOffsetBlock ifNotNil: [:b |
			startOffset := self target
				ifNil: [b numArgs = 0 ifTrue: [b value]]
				ifNotNil: [:obj | b cull: obj]]]! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 6/16/2010 11:55:40.521'!
target

	target ifNil: [target := WeakArray new: 1].
		
	^ target at: 1.! !

!AnimPropertyAnimation methodsFor: 'accessing' stamp: 'mt 6/16/2010 12:04:51.334'!
target: anObject

	target ifNil: [target := WeakArray new: 1].
	target at: 1 put: anObject.! !

!AnimPropertyAnimation methodsFor: 'updating' stamp: 'mt 12/4/2017 11:43'!
updateCurrentValue: aValue

	self target ifNotNil: [:obj |
		obj
			perform: self property asMutator
			with: aValue].! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:05'!
currentEndValue
	^ self endValue + (self endOffset ifNil: [0])! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:05'!
currentStartValue
	^ self startValue + (self startOffset ifNil: [0])! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/11/2017 08:10'!
currentValue

	^ self
		interpolatedFrom: self currentStartValue
		to: self currentEndValue
		progress: self currentLoopTime asFloat / ((self duration - 1) max: 1).! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 6/7/2010 15:27:00.533'!
easingCurve

	^ easingCurve ifNil: [easingCurve := AnimEasingCurve new]! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 12:42:22.17'!
easingCurve: anEasingCurve

	easingCurve := anEasingCurve.! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:24'!
endOffset

	^ endOffset ifNil: [endOffset := endOffsetBlock value]! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
endOffset: anObject

	endOffset := anObject! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
endOffsetBlock

	^ endOffsetBlock! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
endOffsetBlock: anObject

	endOffsetBlock := anObject! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 12:29:23.688'!
endValue

	^ endValue! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 12:29:15.244'!
endValue: anObject

	endValue := anObject.! !

!AnimVariantAnimation methodsFor: 'interpolating' stamp: 'mt 5/14/2010 16:53:16.721'!
interpolatedFrom: fromValue to: toValue progress: aFloat
	"Assumes simple numbers at the moment. Maybe a more complex interpolator could make other types possible."

	^ fromValue + ((toValue - fromValue) *
		(self direction = #backward
			ifTrue: [1.0 - (self easingCurve valueForProgress: aFloat)]
			ifFalse: [self easingCurve valueForProgress: aFloat]))! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:29'!
offset: aValue

	self startOffset: aValue.
	self endOffset: aValue.! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:30'!
offsetBlock: aBlock
	"Will be used to calculate the offset dynamically. See #offset."
	
	self startOffsetBlock: aBlock.
	self endOffsetBlock: aBlock.! !

!AnimVariantAnimation methodsFor: 'control - private' stamp: 'mt 12/6/2017 12:30'!
reset

	super reset.
	
	self startOffsetBlock ifNotNil: [self startOffset: nil].
	self endOffsetBlock ifNotNil: [self endOffset: nil].! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
startOffset

	^ startOffset ifNil: [startOffset := startOffsetBlock value]! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
startOffset: anObject

	startOffset := anObject! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
startOffsetBlock

	^ startOffsetBlock! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 12/6/2017 12:23'!
startOffsetBlock: anObject

	startOffsetBlock := anObject! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 12:29:04.466'!
startValue

	^ startValue! !

!AnimVariantAnimation methodsFor: 'accessing' stamp: 'mt 5/14/2010 12:28:54.496'!
startValue: anObject

	startValue := anObject.! !

!AnimVariantAnimation methodsFor: 'updating' stamp: 'mt 5/14/2010 14:42:02.972'!
updateCurrentValue

	self updateCurrentValue: self currentValue.! !

!AnimVariantAnimation methodsFor: 'updating' stamp: 'mt 5/14/2010 12:40:20.228'!
updateCurrentValue: aValue

	self subclassResponsibility.! !

Object subclass: #AnimComposer
	instanceVariableNames: 'target animation defaultDuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Morphic'!

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/6/2017 11:59'!
animation

	^ animation! !

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/6/2017 11:59'!
animation: anObject

	animation := anObject! !

!AnimComposer methodsFor: 'actions - special' stamp: 'mt 12/11/2017 08:37'!
compose: block

	| composer |
	composer := AnimComposer new
		target: self target;
		defaultDuration: self defaultDuration;
		yourself.
		
	block value: composer.
	
	^ self animations add: composer animation! !

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/8/2017 13:31'!
defaultDuration
	^ defaultDuration ifNil: [500]! !

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/8/2017 13:30'!
defaultDuration: milliseconds
	defaultDuration := milliseconds.! !

!AnimComposer methodsFor: 'actions - special' stamp: 'mt 12/11/2017 08:37'!
do: block
	"Execute some code on the target object. DO NOT closure that object via 'self' in the given block!! Otherweise animGC will get tricky."
	
	"Clean up the block argument to help animGC by discarding some receivers in outer contexts."
	block outerContextsDo: [:ctxt |
		ctxt receiver == self target ifTrue: [ctxt receiver: nil]].
		
	^ self animation add: (
		AnimAnimation new
			name: 'do';
			duration: 1;
			loopCount: 1;
			finishBlock: [(target at: 1) ifNotNil: [:obj | block value: obj]];
			yourself)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
fadeIn

	^ self fadeInIn: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
fadeInIn: milliseconds
	"Warning: Might flicker because it uses multiple cycles."
	
	self target isMorph ifFalse: [Error signal: 'Only morphs can fade.'].
		
	^ self animation add: ((
		AnimAlphaBlendAnimation
			fadeInOn: self target
			duration: milliseconds)
				startBlock: [:morph | morph show];
				yourself)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
fadeOut

	^ self fadeOutIn: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
fadeOutIn: milliseconds
	"Warning: Might flicker because it uses multiple cycles."
	
	self target isMorph ifFalse: [Error signal: 'Only morphs can fade.'].
	
	^ self animation add: ((
		AnimAlphaBlendAnimation
			fadeOutOn: self target
			duration: milliseconds)
				startBlock: [:morph | morph show];
				finishBlock: [:morph | morph hide];
				yourself)! !

!AnimComposer methodsFor: 'initialization' stamp: 'mt 12/6/2017 12:00'!
initialize
	super initialize.
	self animation: AnimCompositeAnimation new.! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveAlong: relativePoints

	^ self
		moveAlong: relativePoints
		in: self defaultDuration * (relativePoints - 1)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveAlong: relativePoints in: milliseconds

	^ self animation add: (
		AnimPathAnimation
			on: self target
			between: relativePoints
			within: milliseconds)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveBy: aPoint

	^ self moveBy: aPoint in: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveBy: aPoint in: milliseconds

	^ self animation add: (
		AnimPropertyAnimation new
			target: self target;
			property: #position;
			startValue: 0;
			endValue: aPoint;
			offsetBlock: [:obj | obj position];
			duration: milliseconds;
			yourself)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveTo: aPoint

	^ self moveTo: aPoint in: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
moveTo: aPoint in: milliseconds

	^ self animation add: (
		AnimPropertyAnimation new
			target: self target;
			property: #position;
			startValue: 0;
			endValue: aPoint;
			startOffsetBlock: [:obj | obj position];
			duration: milliseconds;
			yourself)! !

!AnimComposer methodsFor: 'actions - special' stamp: 'mt 12/11/2017 08:38'!
repeat

	self animation loopCount: -1.
	^ self animation! !

!AnimComposer methodsFor: 'actions - special' stamp: 'mt 12/11/2017 08:38'!
revealText: text in: milliseconds
	"The target morph or one of its submorphs should be able to display the given text."
	
	^ self animation add: (
		AnimParagraphAnimation
			on: self target
			text: text
			duration: milliseconds)! !

!AnimComposer methodsFor: 'actions - special' stamp: 'mt 12/11/2017 08:38'!
revealTextIn: milliseconds
	
	^ self animation add: (
		AnimParagraphAnimation new
			morph: self target;
			duration: milliseconds;
			startValue: 0.0;
			endValue: 1.0;
			yourself)! !

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/9/2017 10:50'!
target

	target ifNil: [target := WeakArray new: 1].
		
	^ target at: 1.! !

!AnimComposer methodsFor: 'accessing' stamp: 'mt 12/9/2017 10:50'!
target: anObject

	target ifNil: [target := WeakArray new: 1].
	target at: 1 put: anObject.! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
wait: milliseconds

	^ self animation add: (
		AnimAnimation new
			name: 'wait';
			duration: milliseconds;
			yourself)! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:36'!
warpBy: aPoint

	^ self warpBy: aPoint in: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:37'!
warpBy: aPoint in: milliseconds

	^ {
		self fadeOutIn: milliseconds // 2.
		self do: [:morph | morph position: morph position + aPoint].
		self fadeInIn: milliseconds // 2 }! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:37'!
warpTo: aPoint

	^ self warpTo: aPoint in: self defaultDuration! !

!AnimComposer methodsFor: 'actions' stamp: 'mt 12/11/2017 08:37'!
warpTo: aPoint in: milliseconds

	^ {
		self fadeOutIn: milliseconds // 2.
		self do: [:morph | morph position: aPoint].
		self fadeInIn: milliseconds // 2 }! !

Object subclass: #AnimEasingCurve
	instanceVariableNames: 'type conversionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Core'!

!AnimEasingCurve commentStamp: 'mt 5/14/2010 19:28:46.069' prior: 0!
I am capable of defining the velocity for the interpolation of an XpfVariantAnimation.!

!AnimEasingCurve class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 14:29:14.198'!
linear

	^ self new
		type: #linear! !

!AnimEasingCurve methodsFor: 'accessing' stamp: 'mt 5/2/2011 20:19:17.89'!
conversionBlock

	^ conversionBlock! !

!AnimEasingCurve methodsFor: 'accessing' stamp: 'mt 5/2/2011 20:19:24.179'!
conversionBlock: aBlock

	conversionBlock := aBlock.! !

!AnimEasingCurve methodsFor: 'testing' stamp: 'mt 5/14/2010 19:29:07.001'!
isLinear

	^ self type = #linear! !

!AnimEasingCurve methodsFor: 'accessing' stamp: 'mt 5/14/2010 14:27:27.368'!
type

	^ type ifNil: [type := #linear]! !

!AnimEasingCurve methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 19:29:25.902'!
type: aType
	"Possible values: #linear."
	
	type := aType.! !

!AnimEasingCurve methodsFor: 'accessing' stamp: 'mt 5/2/2011 20:19:40.876'!
valueForProgress: aFloat
	"Interpolate the given value which should be between 0.0 and 1.0."
	
	^ self conversionBlock
		ifNil: [aFloat "just linear now"]
		ifNotNil: [:b | b value: aFloat]! !

Object subclass: #AnimSignalSpy
	instanceVariableNames: 'object signals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Tests'!

!AnimSignalSpy commentStamp: 'mt 5/14/2010 11:09:41.178' prior: 0!
I am a support class to test the Squeak changed/update mechanism. Should only be used within one thread/process.!

!AnimSignalSpy class methodsFor: 'instance creation' stamp: 'mt 5/14/2010 11:07:44.811'!
on: anObject

	^ self new
		object: anObject! !

!AnimSignalSpy methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:12:02.191'!
lastArguments

	self signals ifEmpty: [Error signal: 'No signals were caught.'].
	^ self signals last second! !

!AnimSignalSpy methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:11:29.309'!
lastSignal

	self signals ifEmpty: [Error signal: 'No signals were caught.'].
	^ self signals last first! !

!AnimSignalSpy methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:11:42.717'!
lastSignalWithArguments

	self signals ifEmpty: [Error signal: 'No signals were caught.'].
	^ self signals last! !

!AnimSignalSpy methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:05:57.559'!
object

	^ object! !

!AnimSignalSpy methodsFor: 'accessing - private' stamp: 'mt 5/14/2010 11:07:25.5'!
object: anObject

	object := anObject.
	object addDependent: self.! !

!AnimSignalSpy methodsFor: 'accessing' stamp: 'mt 5/14/2010 11:06:30.624'!
signals

	^ signals ifNil: [signals := OrderedCollection new]! !

!AnimSignalSpy methodsFor: 'spying' stamp: 'mt 5/14/2010 11:25:24.605'!
update: aSignal

	self signals add: {aSignal. nil}.! !

!AnimSignalSpy methodsFor: 'spying' stamp: 'mt 5/14/2010 11:25:43.285'!
update: aSignal with: oneOrMoreArguments

	self signals add: {aSignal. oneOrMoreArguments}.! !

Object subclass: #AnimSpriteParser
	instanceVariableNames: 'x y bg h w map form endMap endLine newLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Graphics'!

!AnimSpriteParser commentStamp: '<historical>' prior: 0!
Helper class to parse a big form into smaller forms. Uses a background color to detect sprite boundaries. Supports dynamic heights per (horizontal) line and widths per (horizontal) stripe.!

!AnimSpriteParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 08:28'!
parse: aForm

	^ self new
		initializeFrom: aForm;
		parse;
		map! !

!AnimSpriteParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 08:29'!
parse: aForm skip: backgroundColor

	^ self new
		initializeFrom: aForm;
		backgroundColor: backgroundColor;
		parse;
		map! !

!AnimSpriteParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 08:29'!
parseFileNamed: aFileName

	^ self new
		initializeFrom: (Form fromFileNamed: aFileName);
		parse;
		map! !

!AnimSpriteParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 08:30'!
parseFileNamed: aFileName skip: backgroundColor

	^ self new
		initializeFrom: (Form fromFileNamed: aFileName);
		backgroundColor: backgroundColor;
		parse;
		map! !

!AnimSpriteParser methodsFor: 'accessing' stamp: 'mt 12/6/2017 08:27'!
backgroundColor: aColor

	bg := form pixelValueFor: aColor.! !

!AnimSpriteParser methodsFor: 'initialization' stamp: 'mt 12/6/2017 10:56'!
initializeFrom: aForm

	form := aForm.
	bg := form pixelValueAt: 0@0.
	
	self reset.! !

!AnimSpriteParser methodsFor: 'accessing' stamp: 'mt 12/5/2017 09:49'!
map
	^ map! !

!AnimSpriteParser methodsFor: 'parsing' stamp: 'mt 12/6/2017 10:44'!
nextSprite

	| sprite |
	endMap & endLine ifTrue: [^ nil].

	endLine ifTrue: [
		map ifNotEmpty: [
			self skipFilledLines.
			endMap ifTrue: [^ nil]].

		self skipEmptyLines.
		endMap ifTrue: [^ nil].
		
		self peekHeight.
		endLine := false].

	newLine
		ifTrue: [newLine := false]
		ifFalse: [
			self skipFilledStripes.
			endLine ifTrue: [
				newLine := true.
				^ self nextSprite]].
		
	self skipEmptyStripes.
	endLine ifTrue: [
		newLine := true.
		^ self nextSprite].
	
	self peekWidth.
	
	sprite := Form extent: w@h depth: 32.
	form
		displayOn: sprite
		at: (x@y) negated.
	^ sprite! !

!AnimSpriteParser methodsFor: 'parsing' stamp: 'mt 12/6/2017 10:31'!
parse

	| sprite cy |
	[sprite := self nextSprite] whileNotNil: [
		(cy isNil or: [cy ~= y]) ifTrue: [
			map add: OrderedCollection new.
			cy := y].
		map last addLast: sprite].! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:44'!
peekHeight
	"Peek the height of the current line of sprites. Do not change y."
	
	| cy |
	cy := y.
	self skipFilledLines.
	h := y - cy.
	y := cy.
	endMap := false.! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:44'!
peekWidth
	"Peek the width of the current sprite. Do not change x."
	
	| cx |
	cx := x.
	self skipFilledStripes.
	w := x - cx.
	x := cx.
	endLine := false.! !

!AnimSpriteParser methodsFor: 'parsing' stamp: 'mt 12/6/2017 10:55'!
reset

	map := OrderedCollection new.
	x := 0.
	y := 0. "top of current line"
	h := 0. "height of current line/sprite"
	w := 0. "width of current sprite"
	endMap := false.
	endLine := true.
	newLine := true.! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/5/2017 10:37'!
skipEmptyLines
	"Advance the cursor to the next line with sprites."
	
	x := 0.
	[y < form height] whileTrue: [
		(form pixelValueAt: x@y) = bg
			ifFalse: [x := 0. ^ self]
			ifTrue: [
				x = form width
					ifTrue: [y := y + 1. x := 0]
					ifFalse: [x := x + 1]]].
	endMap := true.! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/5/2017 10:37'!
skipEmptyStripes
	"Find the beginning of the next sprite in the current line."

	| cy |
	cy := y.
	[x < form width] whileTrue: [
		(form pixelValueAt: x@cy) = bg
			ifFalse: [cy := y. ^ self]
			ifTrue: [
				cy = (y + h)
					ifTrue: [x := x + 1. cy := y]
					ifFalse: [cy := cy + 1]]].
	endLine := true.! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/5/2017 10:37'!
skipFilledLines
	"Advance the cursor to the next line without sprites."
	
	x := 0.
	[y < form height] whileTrue: [
		(form pixelValueAt: x@y) = bg
			ifFalse: [y := y + 1. x := 0]
			ifTrue: [
				x = form width
					ifTrue: [x := 0. ^ self]
					ifFalse: [x := x + 1]]].
	endMap := true.! !

!AnimSpriteParser methodsFor: 'parsing - private' stamp: 'mt 12/5/2017 10:37'!
skipFilledStripes
	"Find the end of the next sprite in the current line."

	| cy |
	cy := y.
	[x < form width] whileTrue: [
		(form pixelValueAt: x@cy) = bg
			ifFalse: [x := x + 1. cy := y]
			ifTrue: [
				cy = (y + h)
					ifTrue: [cy := y. ^ self]
					ifFalse: [cy := cy + 1]]].
	endLine := true.! !

AnimSpriteParser subclass: #AnimTileParser
	instanceVariableNames: 'tileSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Graphics'!

!AnimTileParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 10:53'!
parse: aForm skip: aPoint

	^ self new
		initializeFrom: aForm;
		tileSize: aPoint;
		parse;
		map! !

!AnimTileParser class methodsFor: 'parsing' stamp: 'mt 12/6/2017 10:53'!
parseFileNamed: aFileName skip: aPoint

	^ self new
		initializeFrom: (Form fromFileNamed: aFileName);
		tileSize: aPoint;
		parse;
		map! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:49'!
peekHeight

	h := self tileSize y.! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:49'!
peekWidth

	w := self tileSize x.! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:51'!
skipEmptyLines
	"Ignore."! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:51'!
skipEmptyStripes
	"Ignore."! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:53'!
skipFilledLines

	x := 0.
	y := y + self tileSize y.
	endMap := y+1 >= form height.! !

!AnimTileParser methodsFor: 'parsing - private' stamp: 'mt 12/6/2017 10:52'!
skipFilledStripes

	x := x + self tileSize x.
	endLine := x+1 >= form width.! !

!AnimTileParser methodsFor: 'accessing' stamp: 'mt 12/6/2017 10:56'!
tileSize
	^ tileSize ifNil: [32@32]! !

!AnimTileParser methodsFor: 'accessing' stamp: 'mt 12/6/2017 10:47'!
tileSize: aPoint
	tileSize := aPoint! !

Object subclass: #AnimTilePainter
	instanceVariableNames: 'palette surface brush'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Animations-Graphics'!

!AnimTilePainter commentStamp: '<historical>' prior: 0!
This is a simple way to make use of a parsed tile map/atlas. It supports putting tiles on a surface to create a simple Morphic hierarchy.

At the time of writing, all tile (morphs) have the same owner, which is the surface. Z order is determined by submorph order.!

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:35'!
brush

	^ brush! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:56'!
brush: anObject

	brush := anObject.
	self changed: #brush.! !

!AnimTilePainter methodsFor: 'painting' stamp: 'mt 12/11/2017 10:33'!
fill: oneOrMoreTiles
	"Randomize the given tiles and fill the entire surface. Remove other tiles first."
	
	| tiles |
	tiles := oneOrMoreTiles isCollection ifTrue: [oneOrMoreTiles] ifFalse: [{oneOrMoreTiles}].

	self surface removeAllMorphs.
	
	1 to: self surfaceExtent y do: [:row |
		1 to: self surfaceExtent x do: [:col |
			self tileAt: col@row put: tiles atRandom]].! !

!AnimTilePainter methodsFor: 'initialize-release' stamp: 'mt 12/11/2017 10:51'!
initialize

	super initialize.
	
	self palette: {{ Form extent: 16@16 depth: 32 }}.
	self surface: Morph new.! !

!AnimTilePainter methodsFor: 'painting' stamp: 'mt 12/11/2017 12:41'!
paintAt: aPoint

	self brush ifNil: [^ self].
	(self tileAt: aPoint) form == self brush ifTrue: [^ self].
	self tileAt: aPoint put: self brush.! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:52'!
palette

	^ palette! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:52'!
palette: anObject

	palette := anObject! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:54'!
paletteTileAt: aPoint
	^ (self palette at: aPoint y) at: aPoint x! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:52'!
surface

	^ surface! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:52'!
surface: anObject

	surface := anObject! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:07'!
surfaceExtent
	^ self surface extent // self tileExtent! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:44'!
surfaceExtent: aPoint
	self surface extent: (aPoint * self tileExtent).
	self changed: #surfaceExtent.! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:50'!
tileAt: aPoint
	"In target."
	
	^ (self tilesAt: aPoint)
		ifEmpty: [nil]
		ifNotEmpty: [:t | t first]! !

!AnimTilePainter methodsFor: 'painting' stamp: 'mt 12/11/2017 10:15'!
tileAt: aPoint put: aTile
	
	^ self surface addMorph: 
		(aTile asMorph
			topLeft: ((aPoint - (1@1)) * self tileExtent) + self surface topLeft;
			yourself)! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:56'!
tileExtent
	^ (self paletteTileAt: 1@1) extent! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 10:40'!
tilesAt: aPoint
	"In target."
	
	^ (self surface morphsAt: ((aPoint - (1@1)) * self tileExtent) + self surface topLeft)
		copyWithout: self surface! !

!AnimTilePainter methodsFor: 'accessing' stamp: 'mt 12/11/2017 09:56'!
tilesUnpacked
	"Creates a list of associations from tile to index in map. Useful for browing and manual editing."
	
	| result |
	result := OrderedCollection new.
	1 to: self palette size do: [:row |
		1 to: (self palette at: row) size do: [:col |
			result add: ((self palette at: row) at: col) -> (col@row)]].
	 ^ result! !

!Object methodsFor: '*animations-support' stamp: 'mt 11/17/2017 17:58'!
compositeAnimations

	self flag: #mutexNeeded.
	^ AnimAnimationRegistry value
		ifNil: [#()]
		ifNotNil: [:registry | registry compositeAnimations select: [:anim | anim includesPropertyAnimationFor: self]]! !
